from zs_utils.api.aliexpress_russia.base_api import AliexpressRussiaAPI


class CreateAliexpressShipmentAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/local-logistic-orders#heading-sozdat-otpravlenie
    """

    resource_method = "/seller-api/v1/logistic-order/create"
    required_params = ["orders"]


class DeleteAliexpressShipmentAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/local-logistic-orders#heading-udalit-otpravlenie
    """

    resource_method = "/seller-api/v1/logistic-order/delete"
    required_params = ["logistic_order_ids"]


class GetAliexpressLabelAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/local-logistic-orders#heading-napechatat-etiketki-dlya-otpravlenii
    """

    resource_method = "/seller-api/v1/labels/orders/get"
    required_params = ["logistic_order_ids"]


# Лист передач (накладная)


class GetAliexpressHandoverListsAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/local-handover-list#heading-poluchit-spisok-listov-peredachi
    """

    resource_method = "/seller-api/v1/handover-list/get-by-filter"
    required_params = ["page_size", "page_number"]
    allowed_params = [
        "statuses",
        "handover_list_ids",
        "logistic_order_ids",
        "gmt_create_from",
        "gmt_create_to",
        "arrival_date_sort",
        "gmt_create_sort",
    ]


class CreateAliexpressHandoverListAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/local-handover-list#heading-sozdat-list-peredachi
    """

    resource_method = "/seller-api/v1/handover-list/create"
    required_params = ["logistic_order_ids"]


class CloseAliexpressHandoverListAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/local-handover-list#heading-zakrit-list-peredachi
    """

    resource_method = "/seller-api/v1/handover-list/transfer"
    required_params = ["handover_list_id"]


class AddAliexpressShipmentToHandoverListAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/local-handover-list#heading-dobavit-otpravlenie-v-list-peredachi
    """

    resource_method = "/seller-api/v1/handover-list/add-logistic-orders"
    required_params = ["handover_list_id", "order_ids"]


class DeleteAliexpressShipmentFromHandoverListAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/local-handover-list#heading-udalit-otpravlenie-iz-lista-peredachi
    """

    resource_method = "/seller-api/v1/handover-list/remove-logistic-orders"
    required_params = ["handover_list_id", "order_ids"]


class GetAliexpressHandoverListLabelAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/local-handover-list#heading-napechatat-list-peredachi
    """

    resource_method = "/seller-api/v1/labels/handover-lists/get"
    required_params = ["handover_list_id"]



from zs_utils.api.apiship.base_api import ApishipAPI


class ApishipLoginApi(ApishipAPI):
    resource_method = "users/login"
    http_method = "POST"
    required_params = ["login", "password"]


class ApishipAutoSignUpAPI(ApishipAPI):
    resource_method = "users/autosignup"
    http_method = "POST"
    required_params = ["login", "password", "email"]



from model_utils import Choices


AspectApplicableTo = Choices(
    ("ITEM", "Групповой товар"),
    ("PRODUCT", "Товар"),
)

AspectDataTypeEnum = Choices(
    ("DATE", "Date"),
    ("NUMBER", "Number"),
    ("STRING", "String"),
    ("STRING_ARRAY", "Array of strings"),
)

AspectModeEnum = Choices(
    ("FREE_TEXT", "Free text"),
    ("SELECTION_ONLY", "Selection only"),
)

AspectUsageEnum = Choices(
    ("RECOMMENDED", "Recommended"),
    ("OPTIONAL", "Optional"),
)

ItemToAspectCardinalityEnum = Choices(
    ("MULTI", "Multi"),
    ("SINGLE", "Single"),
)


GroupAspectType = Choices(
    ("SHARED", "Общий"),
    ("UNIQUE", "Уникальный"),
    ("NOT_USED", "Не используется"),
)



from zs_utils.api.base_api import BaseAPI


class OzonAPI(BaseAPI):
    production_api_url = "https://api-seller.ozon.ru/"
    sandbox_api_url = "https://cb-api.ozonru.me/"
    http_method = "POST"

    def __init__(self, api_key: str, client_id: str, **kwargs):
        super().__init__(**kwargs)

        self.api_key = api_key
        self.client_id = client_id

    @property
    def headers(self) -> dict:
        headers = {
            "Content-Type": "application/json",
            "X-O3-App-Name": "zonesmart",
            "X-O3-App-Version": "zonesmart",
        }
        if self.is_sandbox:
            headers.update(
                {
                    "Api-Key": "0296d4f2-70a1-4c09-b507-904fd05567b9",
                    "Client-Id": "836",
                }
            )
        else:
            headers.update(
                {
                    "Api-Key": self.api_key,
                    "Client-Id": self.client_id,
                }
            )
        return headers



import logging
import requests
import simplejson
from abc import ABC
from unittest.mock import patch
from rest_framework import status
from rest_framework.exceptions import ValidationError
from zs_utils.exceptions import CustomException

from django.utils.translation import gettext as _

from zs_utils.api.base_api import BaseAPI
from zs_utils.api.utils import response_keeper, SessionWithResponseKeeper
from zs_utils.api.constants import API_ERROR_REASONS
from zs_utils.api.services import ApiRequestLogService


__all__ = [
    "APIAction",
    "APIActionError",
]


class APIActionError(CustomException):
    """
    Исключения для APIAction
    """

    pass


class APIAction(ABC):
    """
    Базовый класс экшенов.
    """

    # Атрибуты
    # ------------------------------------------------------------------------------

    # Описание
    description = "Действие с внешним API"

    # Стандартное исключение
    exception_class = APIActionError

    # Предварительные экшены
    ancestor_actions = {}

    # Сущность
    ENTITY_REQUIRED = False
    entity_model = None
    entity_name = None

    # Параметры вызова
    VALIDATE_PARAMS = False
    required_params = []
    allowed_params = []

    # Логирование
    SAVE_REQUEST_LOG = False

    # Таймаут запроса к стороннему API
    RESPONSE_TIMEOUT = 30

    # Логирование запросов (получение requests.Response)
    PATCH_REQUEST_WITH_RESPONSE_KEEPER = False
    REQUEST_PATCH_TARGET = "requests.sessions.Session"  # requests.Session для requests.request()

    # Инициализация
    # ------------------------------------------------------------------------------

    def __init__(
        self,
        instance=None,
        propagate_exception: bool = False,
        raw_results: bool = False,
        **kwargs,
    ):
        self.logger = logging.getLogger()

        self.instance = instance
        if instance:
            self.copy_instance(instance=instance)
        else:
            self.set_action_variables(**kwargs)

        self.propagate_exception = propagate_exception
        self.raw_results = raw_results

        self.validate_action_variables()

    def copy_instance(self, instance, **kwargs) -> None:
        """
        Копирование экземпляра класса instance (вынесение его атрибутов в новый экземпляр)
        """

        self.set_entity(**kwargs)
        if (not self.entity) and (self.entity_model == instance.entity_model):
            self.entity = instance.entity

            if self.entity_name:
                setattr(self, self.entity_name, self.entity)

        # Ссылка на счетчик запросов к внешнему API
        self.request_counter = instance.request_counter

    def set_action_variables(self, **kwargs) -> None:
        """
        Установка атрибутов экшена
        """
        self.set_entity(**kwargs)

        # Инициализация счетчика запросов к внешнему API
        self.request_counter = 0

    def set_entity(self, entity=None, entity_id: str = None, **kwargs) -> None:
        """
        Установка объекта в атрибут
        """
        self.entity = None

        if self.entity_name:
            if self.entity_name in kwargs:
                entity = kwargs[self.entity_name]
            if f"{self.entity_name}_id" in kwargs:
                entity_id = kwargs[f"{self.entity_name}_id"]

        if entity or entity_id:
            if not self.entity_name:
                raise ValueError("Необходимо определить атрибут 'entity_name'.")
            if not self.entity_model:
                raise ValueError("Необходимо определить атрибут 'entity_model'.")

            if entity:
                self.entity = entity
            else:
                self.entity = self.entity_model.objects.get(id=entity_id)

        if self.entity_name:
            setattr(self, self.entity_name, self.entity)

    def validate_action_variables(self) -> None:
        """
        Валидация атрибутов экземпляра
        """
        self.validate_entity()

    def validate_entity(self) -> None:
        """
        Валидация объекта
        """
        if self.entity:
            if not isinstance(self.entity, self.entity_model):
                raise ValueError(
                    f"Параметры 'entity' и '{self.entity_name}' должны быть экземплярами модели '{self.entity_model}'."
                )
        elif self.ENTITY_REQUIRED:
            raise ValueError(
                f"В конструкторе класса {self.__class__.__name__} необходимо задать один из следующих параметров: "
                f"'entity', 'entity_id', '{self.entity_name}', '{self.entity_name}_id'."
            )

    # Вывод сообщения
    # ------------------------------------------------------------------------------

    def _get_message_template(self) -> str:
        """
        Шаблон сообщения
        """
        message = self.description
        if message.endswith("."):
            message = message[:-1]
        return message

    def get_success_internal_message(self) -> str:
        return _("{message_template}: успешно выполнено.").format(message_template=self._get_message_template())

    def get_failure_internal_message(self) -> str:
        return _("{message_template}: возникла ошибка.").format(message_template=self._get_message_template())

    def get_success_message(self, objects) -> str:
        """
        Получение сообщения об успешной отработки экшена
        """

        return self.get_success_internal_message()

    def get_failure_message(self, error: Exception) -> str:
        """
        Получение сообщения об ошибке error при выполнении экшена
        """

        if isinstance(error, CustomException):
            if error.message_dict and error.message_dict.get("external_message"):
                external_message = error.message_dict["external_message"]
            else:
                external_message = error.message

            # Стандартное сообщение для известного типа ошибок
            if (not external_message) and (error.reason in API_ERROR_REASONS):
                external_message = API_ERROR_REASONS[error.reason]

            if not external_message:
                external_message = ""
        else:
            external_message = str(error)

        internal_message = self.get_failure_internal_message()
        if internal_message:
            message = internal_message + " " + external_message
        else:
            message = external_message

        return message

    # Пайплайн
    # ------------------------------------------------------------------------------

    def set_used_action(self, action_class: type["APIAction"], **kwargs) -> "APIAction":
        """
        Инициализация используемого экшена.
        """

        action = action_class(instance=self, propagate_exception=True, **kwargs)

        if not hasattr(self, "used_actions"):
            self.used_actions: list["APIAction"] = []
        self.used_actions.append(action)

        return action

    def set_used_actions(self) -> None:
        """
        Инициализация используемых экшенов.
        """

        pass

    def execute_ancestor_actions(self) -> None:
        """
        Выполнение предварительных экшенов и передача их результатов в качестве параметров текущего экшена.
        """

        for results_name, action_class in self.ancestor_actions.items():
            if not self.params.get(results_name):
                is_success, message, objects = action_class(instance=self, propagate_exception=True)(**self.params)
                self.params[results_name] = objects.get("results", objects)

    def __call__(self, **kwargs) -> tuple:
        """
        Запуск экшена (контроллер)
        """
        is_success = False
        message = None
        objects = {}

        try:
            # Инициализация используемых экшенов
            self.set_used_actions()

            # Обработка входных параметров
            self.params = self.get_params(**kwargs)

            # Действия перед началом выполнения данного экшена
            self.before_request()

            # Выполнение предварительных экшенов и передача их результатов в качестве параметров данного экшена
            self.execute_ancestor_actions()

            # Выполнение данного экшена
            objects: dict = self.run_action(**self.params)
            if not objects:
                objects = {"results": {}, "response": None}

            # Действия после успешного выполнения данного экшена
            self.success_callback(objects=objects, **self.params)
        except CustomException as error:
            response: requests.Response = error.response

            if not error.reason:
                error.reason = API_ERROR_REASONS.unknown

            is_success = False
            message: str = self.get_failure_message(error=error)
            objects = {
                "results": {},
                "errors": error,
                "error_reason": error.reason,
                "response": response,
            }

            # Действия после неудачного выполнения данного экшена
            self.failure_callback(objects)

            if self.propagate_exception:
                raise
        except ValidationError as error:  # TODO: не обрабатывать ValidationError?
            is_success = False
            message: str = self.get_failure_message(error=error)
            objects = {
                "results": {},
                "errors": error,
                "error_reason": API_ERROR_REASONS.data_validation,
                "response": None,
            }

            # Действия после неудачного выполнения данного экшена
            self.failure_callback(objects)

            if self.propagate_exception:
                raise
        else:
            is_success = True
            message = self.get_success_message(objects)

        if is_success:
            self.logger.info(message)
        else:
            self.logger.warning(message)

        return is_success, message, objects

    @classmethod
    def simple_run(cls, **kwargs) -> dict:
        """
        Вызов конструктора с propagate_exception=True, а затем вызов __call__ и возврат objects["results"].
        """

        kwargs["propagate_exception"] = True
        action = cls(**kwargs)
        return action.__call__(**kwargs)[2].get("results", {})

    # Методы, поддерживающие переопределение
    # ------------------------------------------------------------------------------

    def before_request(self, **kwargs) -> None:
        """
        Действия перед выполнением запроса
        """
        pass

    def format_success_results(self, results: dict, **kwargs) -> dict:
        """
        Форматирование результатов после успешного выполнения данного экшена.
        """

        return results

    def success_callback(self, objects: dict, **kwargs) -> None:
        """
        Действия после успешного выполнения данного экшена.
        """

        if (not self.raw_results) and isinstance(objects, dict) and ("results" in objects):
            objects["results"] = self.format_success_results(results=objects["results"], objects=objects)

    def failure_callback(self, objects: dict, **kwargs) -> None:
        """
        Действия после неудачного выполнения данного экшена.
        """

        pass

    def get_params(self, **kwargs) -> dict:
        """
        Определение параметров вызова экшена.
        """

        return kwargs

    @property
    def possible_params(self):
        """
        Возможные параметры (required_params + allowed_params)
        """
        return self.required_params + self.allowed_params + ["size", "page"]  # TODO: где нужны size и page?

    def clean_api_request_params(self, raw_params: dict) -> dict:
        """
        Валидация и отчистка параметров запроса к API
        """
        # Проверка наличия обязательных параметров
        for param in self.required_params:
            if raw_params.get(param) is None:
                raise self.exception_class(f"Обязательный параметр '{param}' не задан.")

        # Фильтрация допустимых параметров
        params = {param: raw_params[param] for param in self.possible_params if param in raw_params}

        return params

    def get_api_class_init_params(self, **kwargs) -> dict:
        """
        Получение параметров для инициализации API класса
        """
        raise NotImplementedError("Необходимо определить метод 'get_api_class_init_params'.")

    def get_api(self, **kwargs) -> BaseAPI:
        """
        Получение API класса
        """
        if not getattr(self, "api_class", None):
            raise NotImplementedError("Необходимо определить атрибут 'api_class'.")
        init_params: dict = self.get_api_class_init_params(**kwargs)
        init_params["response_timeout"] = self.RESPONSE_TIMEOUT
        return self.api_class(**init_params)

    def make_request(self, **kwargs) -> dict:
        """
        Исполнение запроса через API класс к API сервиса
        """
        self.api: BaseAPI = self.get_api(**kwargs)

        try:
            response = self.api.make_request(**kwargs)
        except (requests.exceptions.Timeout, requests.exceptions.ConnectTimeout):
            raise self.exception_class(reason=API_ERROR_REASONS.timeout)

        results = self.get_response_results(response=response)

        return {"results": results, "response": response}

    def get_response_results(self, response: requests.Response) -> dict:
        """
        Извлечение результатов выполнения запроса из response
        """
        if response is not None:
            try:
                return response.json()
            except simplejson.JSONDecodeError:
                pass
        return {}

    def run_action(self, **kwargs) -> dict:
        """
        Запрос к API сервиса с последующей обработкой ответа
        """
        if self.VALIDATE_PARAMS:
            params = self.clean_api_request_params(raw_params=kwargs)
        else:
            params = kwargs

        # Инкрементирование счетчика запросов к API маркетплейса
        self.incr_request_counter()

        exception = None
        try:
            if self.PATCH_REQUEST_WITH_RESPONSE_KEEPER:
                with patch(self.REQUEST_PATCH_TARGET, SessionWithResponseKeeper):
                    response_data: dict = self.make_request(**params)
                # Извлекаем из декоратора сохраненный response
                response_attr = "raw_response"
                response: requests.Response = getattr(response_keeper, response_attr, None)
                if response is not None:
                    response_data["response"] = response
                    setattr(response_keeper, response_attr, None)
            else:
                response_data: dict = self.make_request(**params)
        except self.exception_class as error:
            response: requests.Response = error.response
            if response is None:
                results = {}
            else:
                results = self.get_response_results(response=response)
            if (not results) and error.message_dict:
                results = error.message_dict
            exception = error
        else:
            results: dict = response_data["results"]
            response: requests.Response = response_data.get("response")

        response_is_success: bool = self.response_is_success(results=results, response=response, exception=exception)
        if response_is_success:
            results: dict = self.format_success_response_results(results=results, response=response)
            exception = None
        else:
            error_message = self.get_error_message(results=results, response=response)
            if (not error_message) and exception:
                error_message = exception.message

            if exception is None:
                exception = self.exception_class(message=error_message, response=response)

            # Определение типа ошибки
            if not exception.reason:
                exception.reason = self.get_error_reason(
                    results=results,
                    response=response,
                    error_message=error_message,
                )

        # Сохранение лога
        if (not response_is_success) or self.SAVE_REQUEST_LOG:
            try:
                self.save_request_log(results=results, response=response)
            except Exception as error:
                self.logger.critical(f"Не удалось сохранить лог запроса к внешнему API: {str(error)}")

        if exception is not None:
            raise exception

        return {"results": results, "response": response}

    def incr_request_counter(self) -> None:
        """
        Увеличение счётчика запросов
        """
        self.request_counter += 1
        if self.instance:
            self.instance.incr_request_counter()

    def response_is_success(
        self,
        results: dict,
        response: requests.Response,
        exception: CustomException | None = None,
    ) -> bool:
        """
        Проверка, что запрос успешный
        """
        if exception is not None:
            return False
        if response is not None:
            return status.is_success(response.status_code)
        return True

    def format_success_response_results(self, results: dict, response: requests.Response) -> dict:
        """
        Форматирование данных ответа успешного запроса
        """
        return results

    def get_error_message(self, results: dict, response: requests.Response) -> str:
        """
        Получение сообщения об ошибке из ответа response или results
        """
        return ""

    def get_error_reason(self, results: dict, response: requests.Response, error_message: str) -> API_ERROR_REASONS:
        """
        Получение причины ошибки
        """

        if (response is not None) and getattr(response, "status_code", None):
            code: int = response.status_code
            if str(code).startswith("5"):
                return API_ERROR_REASONS.system
            elif code in [401, 403]:
                return API_ERROR_REASONS.invalid_token
            elif code == 404:
                return API_ERROR_REASONS.object_not_found
            elif response.status_code == 429:
                return API_ERROR_REASONS.request_limit

    def get_apirequest_log_extra_data(self) -> dict:
        return {}

    def save_request_log(self, results: dict, response: requests.Response):
        if (response is not None) and isinstance(response, requests.Response):
            ApiRequestLogService.save_api_request_log_by_response(
                response=response,
                **self.get_apirequest_log_extra_data(),
            )



import django.utils.timezone
import model_utils.fields
import uuid

from django.conf import settings
from django.db import migrations, models

from zs_utils.file_utils import get_support_file_upload_path


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="SupportTicket",
            fields=[
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                (
                    "id",
                    model_utils.fields.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                ("number", models.CharField(max_length=16, verbose_name="Номер обращения")),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("PENDING", "Ожидает рассмотрения"),
                            ("OPEN", "Открыто"),
                            ("CLOSED_BY_USER", "Закрыто пользователем"),
                            ("CLOSED_BY_MANAGER", "Закрыто менеджером"),
                            ("CLOSED_AUTO", "Закрыто из-за отсутствия активности"),
                        ],
                        max_length=30,
                        verbose_name="Статус",
                    ),
                ),
                (
                    "question_type",
                    models.CharField(max_length=50, verbose_name="Тип вопроса"),
                ),
                ("subject", models.CharField(max_length=300, verbose_name="Тема")),
                (
                    "manager",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Менеджер",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="tickets",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Создатель",
                    ),
                ),
            ],
            options={
                "ordering": ["-created"],
            },
        ),
        migrations.CreateModel(
            name="SupportTicketMessage",
            fields=[
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                (
                    "id",
                    model_utils.fields.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                ("text", models.TextField(max_length=1500, verbose_name="Сообщение")),
                ("is_system", models.BooleanField(blank=True, default=False, verbose_name="Системное сообщение")),
                ("is_viewed", models.BooleanField(blank=True, default=False, verbose_name="Просмотрено получателем")),
                (
                    "sender",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Автор сообщения",
                    ),
                ),
                (
                    "ticket",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="messages",
                        related_query_name="message",
                        to="support.supportticket",
                        verbose_name="Тикет",
                    ),
                ),
            ],
            options={
                "ordering": ["-created"],
            },
        ),
        migrations.CreateModel(
            name="SupportTicketMessageFile",
            fields=[
                (
                    "id",
                    model_utils.fields.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                (
                    "file",
                    models.FileField(
                        upload_to=get_support_file_upload_path,
                        verbose_name="Приложенный файл",
                    ),
                ),
                (
                    "ticket_message",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="files",
                        related_query_name="file",
                        to="support.supportticketmessage",
                        verbose_name="Сообщение",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Создатель",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from zs_utils.api.ozon.base_api import OzonAPI


class GetReportInfoAPI(OzonAPI):
    resource_method = "v1/report/info"
    required_params = ["code"]


class CreateStockReportAPI(OzonAPI):
    resource_method = "v1/report/stock/create"



from zs_utils.api.etsy.base_api import EtsyAPI


class GetEtsyAccountInfo(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getUser
    """

    http_method = "GET"
    resource_method = "users/{user_id}"
    required_params = ["user_id"]



import requests

from django.core.cache import cache
from django.conf import settings
from django.utils.translation import gettext as _

from zs_utils.exceptions import CustomException


class CurrencyServiceError(CustomException):
    pass


def _convert_currency_by_coinlayer(from_currency: str, to_currency: str, amount: int = 1) -> float:
    """
    Docs: https://coinlayer.com/documentation
    Конвертация валют через сервис coinlayer
    """

    token = settings.COINLAYER_API_TOKEN
    url = (
        f"http://api.coinlayer.com/api/convert?access_key={token}&from={from_currency}&to={to_currency}&amount={amount}"
    )
    data = requests.get(url=url).json()

    if not data.get("success", True):
        message = _("Не удалось конвертировать валюту ({from_currency}-->{to_currency}): {error}").format(
            from_currency=from_currency,
            to_currency=to_currency,
            error=data.get("error", {}).get("info", "System error"),
        )
        raise CurrencyServiceError(message=message)

    return data["result"]


def _get_exchange_rate_cache_key(from_currency: str, to_currency: str):
    """
    Получения ключа кэша, в котором хранятся курсы валют
    """
    sorted_currencies = sorted([from_currency, to_currency])
    return f"exchange_rate_{sorted_currencies[0]}_{sorted_currencies[1]}"


def _get_exchange_rate_from_cache(from_currency: str, to_currency: str) -> float:
    """
    Получение курса валют из кэша
    """
    value = cache.get(key=_get_exchange_rate_cache_key(from_currency=from_currency, to_currency=to_currency))
    if value and (from_currency > to_currency):
        value = 1 / value
    return value


def _save_exchange_rate_in_cache(from_currency: str, to_currency: str, value: float) -> None:
    """
    Сохранение курса валют в кэш
    """
    assert value
    if from_currency > to_currency:
        value = 1 / value
    timeout = 0.5 * 60 * 60  # 0.5 часов
    cache.set(
        _get_exchange_rate_cache_key(from_currency=from_currency, to_currency=to_currency),
        value,
        timeout=timeout,
    )


def convert_amount(
    from_currency: str,
    to_currency: str,
    value: float,
    use_cache: bool = True,
) -> float:
    """
    Конвертация переданных валют, с возможностью использования кэша
    """
    if from_currency == "RUR":
        from_currency = "RUB"
    if to_currency == "RUR":
        to_currency = "RUB"

    # Тривиальный случай
    if (from_currency == to_currency) or (value == 0):
        return value

    coef = None

    # Получение сохраненного в кэше курса
    if use_cache:
        coef = _get_exchange_rate_from_cache(from_currency=from_currency, to_currency=to_currency)

    if not coef:
        coef = _convert_currency_by_coinlayer(
            from_currency=from_currency,
            to_currency=to_currency,
            amount=1,
        )
        # Сохранение курса в кэш
        _save_exchange_rate_in_cache(from_currency=from_currency, to_currency=to_currency, value=coef)

    return round(value * coef, 5)


@classmethod
def is_greater(value1: float, currency1: str, value2: float, currency2: str) -> bool:
    """
    Метод для сравнения суммы в разных валютах
    """
    return convert_amount(from_currency=currency1, to_currency=currency2, value=value1) > value2



from . import fields

from .abstract import AbstractModel
from .feed import Feed
from .shop import Shop
from .currency import Currency
from .category import Category
from .option import Option
from .price import Price, OldPrice
from .offers import (
    SimplifiedOffer,
    ArbitraryOffer,
    BookOffer,
    AudioBookOffer,
    MusicVideoOffer,
    MedicineOffer,
    EventTicketOffer,
    AlcoholOffer,
)
from .parameter import Parameter
from .condition import Condition
from .dimensions import Dimensions
from .age import Age
from .gift import Gift
from .promo import Promo, Purchase, Product, PromoGift


__all__ = [
    "fields",
    "AbstractModel",
    "Feed",
    "Shop",
    "Currency",
    "Category",
    "Option",
    "Price",
    "SimplifiedOffer",
    "ArbitraryOffer",
    "BookOffer",
    "AudioBookOffer",
    "MusicVideoOffer",
    "MedicineOffer",
    "EventTicketOffer",
    "AlcoholOffer",
    "Parameter",
    "Condition",
    "Dimensions",
    "Age",
    "Gift",
    "Promo",
    "Purchase",
    "Product",
    "PromoGift",
    "Outlet",
]



from .category import *
from .dropshipping import *
from .freight_template import *
from .logistics import *
from .merchant import *
from .order import *
from .product import *
from .service_template import *



from zs_utils.api.etsy.base_api import EtsyAPI


class GetEtsyCarriers(EtsyAPI):
    """
    Docs:
    https://developers.etsy.com/documentation/reference/#operation/getShippingCarriers
    """

    description = "Получение списка курьерских служб Etsy"
    http_method = "GET"
    resource_method = "shipping-carriers"
    required_params = ["origin_country_iso"]



from rest_framework_nested import routers

from zs_utils.support import views


app_name = "support"

router = routers.SimpleRouter()
router.register("ticket/file", views.BaseSupportTicketMessageFileView, basename="ticket_file")
router.register("ticket", views.BaseSupportTicketView, basename="ticket")
ticket_router = routers.NestedDefaultRouter(router, "ticket", lookup="ticket")
ticket_router.register("message", views.BaseSupportTicketMessageView, basename="ticket-message")

urlpatterns = []
urlpatterns += router.urls
urlpatterns += ticket_router.urls



from zs_utils.api.etsy.base_api import EtsyAPI


class CreateEtsyShippingProfileDestination(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/createListingShippingProfileDestination
    """

    http_method = "POST"
    resource_method = "shops/{shop_id}/shipping-profiles/{shipping_profile_id}/destinations"
    required_params = [
        "shop_id",
        "shipping_profile_id",
        "primary_cost",
        "secondary_cost",
    ]
    allowed_params = [
        "destination_country_iso",
        "destination_region",
        "shipping_carrier_id",
        "mail_class",
        "min_delivery_days",
        "max_delivery_days",
    ]


class GetEtsyShippingProfileDestinationList(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getListingShippingProfileDestinationsByShippingProfile
    """

    http_method = "POST"
    resource_method = "shops/{shop_id}/shipping-profiles/{shipping_profile_id}/destinations"
    required_params = ["shop_id", "shipping_profile_id"]


class UpdateEtsyShippingProfileDestination(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/updateListingShippingProfileDestination
    """

    http_method = "PUT"
    resource_method = (
        "shops/{shop_id}/shipping-profiles/{shipping_profile_id}/destinations/{shipping_profile_destination_id}"
    )
    required_params = [
        "shop_id",
        "shipping_profile_id",
        "shipping_profile_destination_id",
    ]
    allowed_params = [
        "primary_cost",
        "secondary_cost",
        "destination_country_iso",
        "destination_region",
        "shipping_carrier_id",
        "mail_class",
        "min_delivery_days",
        "max_delivery_days",
    ]


class DeleteEtsyShippingProfileDestination(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/deleteListingShippingProfileDestination
    """

    http_method = "DELETE"
    resource_method = (
        "shops/{shop_id}/shipping-profiles/{shipping_profile_id}/destinations/{shipping_profile_destination_id}"
    )
    required_params = [
        "shop_id",
        "shipping_profile_id",
        "shipping_profile_destination_id",
    ]



from .abstract import AbstractModel, XMLElement


class Dimensions(AbstractModel):
    """
    Offer dimensions model (length, width, height) in the package in
    centimeters.
    """

    __slots__ = ["_length", "_width", "_height"]

    def __init__(self, length, width, height):
        self.length = length
        self.width = width
        self.height = height

    @property
    def length(self) -> float:
        return float(self._length)

    @length.setter
    def length(self, value):
        self._length = self._is_valid_float(value, "length")

    @property
    def width(self) -> float:
        return float(self._width)

    @width.setter
    def width(self, value):
        self._width = self._is_valid_float(value, "width")

    @property
    def height(self) -> float:
        return float(self._height)

    @height.setter
    def height(self, value):
        self._height = self._is_valid_float(value, "height")

    def create_dict(self, **kwargs) -> dict:
        return dict(length=self.length, width=self.width, height=self.height)

    def create_xml(self, **kwargs) -> XMLElement:
        el = XMLElement("dimensions")
        el.text = "/".join([self._length, self._width, self._height])
        return el

    @staticmethod
    def from_xml(el: XMLElement) -> "Dimensions":
        return Dimensions(*el.text.split("/"))



from yookassa import Payment

from zs_utils.api.yookassa.base_action import YooKassaAction
from zs_utils.api.yookassa import services


__all__ = [
    "RemoteCreateYooKassaPayment",
]


class RemoteCreateYooKassaPayment(YooKassaAction):
    description = "Создание платежа в Яндекс.Кассе"
    USER_REQUIRED = True
    api_method = Payment.create
    required_params = ["params"]

    def get_params(
        self,
        amount: float,
        currency: str,
        description: str = None,
        payment_method_id: str = None,
        receipt: dict = None,
        extra_metadata: dict = None,
        return_url: str = None,
        **kwargs,
    ):
        # Описание платежа
        if description and (len(description) > 128):
            description = description[:125] + "..."

        # Метаданные
        metadata = {
            "user_id": self.user.id,
        }
        if extra_metadata:
            metadata.update(extra_metadata)
        if len(metadata.keys()) > 16:
            raise self.exception_class("Словарь 'metadata' должен содержать не более 16 ключей.")

        kwargs["params"] = {
            "amount": {"value": amount, "currency": currency},
            "description": description,
            "capture": True,
            "metadata": metadata,
            "confirmation": {
                "type": "redirect",  # или "embedded"
                "return_url": return_url,
                "enforce": False,
            },
        }

        if payment_method_id:
            # Автоплатеж
            kwargs["params"]["payment_method_id"] = payment_method_id
        else:
            # Подключение карты
            kwargs["params"].update(
                {
                    "save_payment_method": True,
                    "payment_method_data": {"type": "bank_card"},
                }
            )

        if not receipt:
            kwargs["params"]["receipt"] = services.BaseYooKassaService.get_receipt_template(
                product_description=description,
                product_price=amount,
                product_price_currency=currency,
                user=self.user,
            )

        return super().get_params(**kwargs)

    def success_callback(self, objects: dict, **kwargs) -> None:
        super().success_callback(objects, **kwargs)

        data = objects["results"]
        objects["results"] = {
            "backend": "yandex_checkout",
            "status": data["status"],
            "confirmation_token": None,
            "redirect_to_url": None,
        }
        if data.get("confirmation"):
            if data["confirmation"]["type"] == "redirect":
                objects["results"]["redirect_to_url"] = data["confirmation"]["confirmation_url"]
            elif data["confirmation"]["type"] == "embedded":
                objects["results"]["confirmation_token"] = data["confirmation"]["confirmation_token"]



from zs_utils.api.ebay.base_api import EbayAPI


class CreateReturnPolicy(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/account/resources/return_policy/methods/createReturnPolicy
    """

    http_method = "POST"
    resource_method = "sell/account/v1/return_policy"


class DeleteReturnPolicy(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/account/resources/return_policy/methods/deleteReturnPolicy
    """

    http_method = "DELETE"
    resource_method = "sell/account/v1/return_policy/{return_policy_id}"
    required_params = ["return_policy_id"]


class GetEbayReturnPolicy(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/account/resources/return_policy/methods/getReturnPolicy
    """

    http_method = "GET"
    resource_method = "sell/account/v1/return_policy/{return_policy_id}"
    required_params = ["return_policy_id"]


class GetReturnPolicies(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/account/resources/return_policy/methods/getReturnPolicies
    """

    http_method = "GET"
    resource_method = "sell/account/v1/return_policy"
    required_params = ["marketplace_id"]


class GetReturnPolicyByName(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/account/resources/return_policy/methods/getReturnPolicyByName
    """

    http_method = "GET"
    resource_method = "sell/account/v1/return_policy/get_by_policy_name"
    required_params = ["marketplace_id", "name"]


class UpdateEbayReturnPolicy(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/account/resources/return_policy/methods/updateReturnPolicy
    """

    http_method = "PUT"
    resource_method = "sell/account/v1/return_policy/{return_policy_id}"
    required_params = ["return_policy_id"]



import requests
import simplejson
from urllib.parse import parse_qs, urlparse
from rest_framework import status

from django.apps import apps
from django.conf import settings

from zs_utils.api import models


__all__ = [
    "ApiRequestLogService",
]


class ApiRequestLogService:
    """
    Сервис для работы с моделью APIRequestLog
    """

    @classmethod
    def get_request_log_model(cls) -> type[models.AbstractAPIRequestLog] | None:
        if getattr(settings, "API_REQUEST_LOG_MODEL", None):
            app_label, model_name = settings.API_REQUEST_LOG_MODEL.split(".")
            return apps.get_model(app_label=app_label, model_name=model_name)
        else:
            return models.APIRequestLog

    @classmethod
    def save_api_request_log(
        cls,
        status_code: int,
        url: str,
        method: str,
        request_headers: dict = None,
        response_headers: dict = None,
        request_body: dict = None,
        response_body: dict = None,
        response_time: float = None,
        user: settings.AUTH_USER_MODEL = None,
        save_if_is_success: bool = True,
        **extra_fields,
    ) -> models.AbstractAPIRequestLog:
        """
        Создание экземпляра модели AbstractAPIRequestLog по переданным данным (сохранение данных)
        """
        if (not save_if_is_success) and status.is_success(status_code):
            return None

        request_log_model = cls.get_request_log_model()
        if not request_log_model:
            raise ValueError("Необходимо задать настройку API_REQUEST_LOG_MODEL (путь к модели).")

        return request_log_model.objects.create(
            user=user,
            # Данные запроса
            url=url,
            method=method,
            params=parse_qs(urlparse(url).query),
            request_headers=request_headers,
            request_body=request_body,
            # Данные ответа
            response_time=response_time,
            status_code=status_code,
            response_headers=response_headers,
            response_body=response_body,
            **extra_fields,
        )

    @classmethod
    def save_api_request_log_by_response(
        cls,
        response: requests.Response,
        user: settings.AUTH_USER_MODEL = None,
        save_if_is_success: bool = True,
        **extra_fields,
    ) -> models.AbstractAPIRequestLog:
        """
        Создание экземпляра модели AbstractAPIRequestLog по переданному requests.Response (сохранение данных)
        """
        request = response.request

        request_body = request.body

        if isinstance(request_body, bytes):
            try:
                request_body = request_body.decode()
            except UnicodeDecodeError:
                request_body = str(request_body)

        if isinstance(request_body, str):
            try:
                request_body = simplejson.loads(request_body)
            except simplejson.JSONDecodeError:
                pass

        try:
            response_body = response.json()
        except simplejson.JSONDecodeError:
            response_body = None

        return cls.save_api_request_log(
            user=user,
            status_code=response.status_code,
            url=request.url,
            method=request.method,
            request_headers=dict(request.headers),
            response_headers=dict(response.headers),
            request_body=request_body,
            response_body=response_body,
            response_time=response.elapsed.microseconds / 1000,
            save_if_is_success=save_if_is_success,
            **extra_fields,
        )



from model_utils.models import TimeStampedModel, UUIDModel
from django.db import models
from django.contrib.auth import get_user_model

from zs_utils.file_utils import get_support_file_upload_path
from zs_utils.support import constants


__all__ = [
    "SupportTicket",
    "SupportTicketMessage",
    "SupportTicketMessageFile",
]

User = get_user_model()


class SupportTicket(TimeStampedModel, UUIDModel):
    """
    Данные тикета (запросов в поддержку)
    """
    number = models.TextField(verbose_name="Номер обращения")
    subject = models.CharField(max_length=300, verbose_name="Тема")
    status = models.TextField(choices=constants.SUPPORT_TICKET_STATUSES, verbose_name="Статус")
    question_type = models.TextField(verbose_name="Тип вопроса")
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name="tickets", verbose_name="Создатель")
    manager = models.ForeignKey(User, blank=True, null=True, on_delete=models.SET_NULL, verbose_name="Менеджер")

    class Meta:
        ordering = ["-created"]

    @property
    def client_status(self) -> constants.SUPPORT_TICKET_CLIENT_STATUSES:
        if self.status in constants.SUPPORT_TICKET_ACTIVE_STATUSES:
            if self.manager:
                last_message = self.last_message
                if last_message and (last_message.sender == self.manager):
                    return constants.SUPPORT_TICKET_CLIENT_STATUSES.RESPONDED
            return constants.SUPPORT_TICKET_CLIENT_STATUSES.PENDING
        else:
            return constants.SUPPORT_TICKET_CLIENT_STATUSES.CLOSED

    @property
    def is_active(self):
        """
        Тикет активный
        """
        return self.status in constants.SUPPORT_TICKET_ACTIVE_STATUSES

    @property
    def unread_messages(self) -> dict:
        """
        Не прочтённые сообщения
        """
        unread = self.messages.filter(is_system=False, is_viewed=False)
        return {
            "user": unread.exclude(sender=self.user).count(),
            "manager": unread.filter(sender=self.user).count(),
        }

    @property
    def last_message(self):
        """
        Последнее сообщение в данном обращении
        """
        return self.messages.filter(is_system=False).order_by("-created").first()


class SupportTicketMessage(TimeStampedModel, UUIDModel):
    """
    Данные сообщения тикета
    """
    text = models.TextField(max_length=constants.MAX_TEXT_LENGTH, null=True, verbose_name="Сообщение")
    is_system = models.BooleanField(verbose_name="Системное сообщение")
    is_viewed = models.BooleanField(default=False, verbose_name="Просмотрено получателем")
    ticket = models.ForeignKey(
        "SupportTicket",
        on_delete=models.CASCADE,
        related_name="messages",
        related_query_name="message",
        verbose_name="Тикет",
    )
    sender = models.ForeignKey(
        to=User,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        verbose_name="Автор сообщения",
    )

    class Meta:
        ordering = ["-created"]

    @property
    def is_from_manager(self) -> bool:
        """
        Сообщение от менеджера
        """
        return self.ticket.manager and (self.sender == self.ticket.manager)

    @property
    def is_from_client(self) -> bool:
        """
        Сообщение от пользователя
        """
        return self.sender == self.ticket.user

    @property
    def recipient(self):
        """
        Получатель сообщения
        """
        if self.sender == self.ticket.user:
            return self.ticket.manager
        else:
            return self.ticket.user


class SupportTicketMessageFile(UUIDModel):
    """
    Данные прикреплённого файла к сообщению тикета
    """
    file = models.FileField(upload_to=get_support_file_upload_path, verbose_name="Приложенный файл")
    ticket_message = models.ForeignKey(
        "SupportTicketMessage",
        on_delete=models.CASCADE,
        related_name="files",
        related_query_name="file",
        verbose_name="Сообщение",
        null=True,
        blank=True,
    )
    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        related_name="+",
        verbose_name="Пользователь",
        null=True,
        blank=True,
    )

    @property
    def is_used(self) -> bool:
        if not self.ticket_message:
            raise NotImplemented
        return bool(self.ticket_message_id)

    @property
    def name(self) -> str:
        return self.file.name.split("/")[-1]



from .abstract import AbstractModel, XMLElement


class Outlet(AbstractModel):
    """
    Используйте элемент outlets, чтобы указать:
    1) количество товара, доступное в конкретной точке продаж;
    2) цены, которые действуют в конкретных точках и отличаются от цены в большинстве точек.

    Docs:
    https://yandex.ru/support/edadeal-partner/pricelist/outlets.html
    """

    __slots__ = ["id", "instock"]

    def __init__(self, id: str = None, instock: int = 0):
        self.id = id
        self.instock = instock

    def create_dict(self, **kwargs) -> dict:
        return dict(id=self.id, instock=self.instock)

    def create_xml(self, **kwargs) -> XMLElement:
        attribs = {"id": self.id, "instock": self.instock}
        el = XMLElement("outlet", attrib=attribs)
        return el

    @staticmethod
    def from_xml(el: XMLElement) -> "Outlet":
        return Outlet(**el.attrib)



from .abstract import AbstractModel, XMLElement

from zs_utils.yandex_market_language.exceptions import ValidationError


UNIT_CHOICES = ("year", "month")
YEAR_CHOICES = [y for y in range(0, 19, 6)]
MONTH_CHOICES = [m for m in range(0, 13)]


class Age(AbstractModel):
    """
    Age category of the offer.
    """

    __slots__ = ["_unit", "_value"]

    def __init__(self, unit: str, value):
        self.unit = unit
        self.value = value

    @property
    def unit(self) -> str:
        return self._unit

    @unit.setter
    def unit(self, value):
        if value and value not in UNIT_CHOICES:
            raise ValidationError("unit must be a valid choice: {c}".format(c=", ".join(UNIT_CHOICES)))
        self._unit = value

    @property
    def value(self) -> int:
        return int(self._value)

    @value.setter
    def value(self, v):
        try:
            v = int(v)

            # Check valid choices for specified unit
            not_valid, choices = False, None
            if self.unit == "year":
                if v not in YEAR_CHOICES:
                    not_valid, choices = True, YEAR_CHOICES
            else:
                if v not in MONTH_CHOICES:
                    not_valid, choices = True, MONTH_CHOICES

            # Raise ValidationError if choice is not valid
            if not_valid:
                raise ValidationError(
                    "value for unit 'year' must be a valid choice: " "{c}".format(c=", ".join(str(c) for c in choices))
                )

            self._value = str(v)
        except (TypeError, ValueError):
            raise ValidationError("value must be a valid int")

    def create_dict(self, **kwargs) -> dict:
        return dict(unit=self.unit, value=self.value)

    def create_xml(self, **kwargs) -> XMLElement:
        attribs = {}
        if self._unit:
            attribs["unit"] = self._unit
        el = XMLElement("age", attribs)
        el.text = self._value
        return el

    @staticmethod
    def from_xml(el: XMLElement) -> "Age":
        return Age(unit=el.attrib.get("unit"), value=el.text)



from model_utils import Choices


__all__ = [
    "USA_STATES",
    "USA_STATE_CODE_TO_NAME",
    "USA_STATE_NAME_TO_CODE",
]


USA_STATES = Choices(
    ("Alabama", "AL"),
    ("Alaska", "AK"),
    ("American Samoa", "AS"),
    ("Arizona", "AZ"),
    ("Arkansas", "AR"),
    ("California", "CA"),
    ("Colorado", "CO"),
    ("Connecticut", "CT"),
    ("Delaware", "DE"),
    ("District of Columbia", "DC"),
    ("Florida", "FL"),
    ("Georgia", "GA"),
    ("Guam", "GU"),
    ("Hawaii", "HI"),
    ("Idaho", "ID"),
    ("Illinois", "IL"),
    ("Indiana", "IN"),
    ("Iowa", "IA"),
    ("Kansas", "KS"),
    ("Kentucky", "KY"),
    ("Louisiana", "LA"),
    ("Maine", "ME"),
    ("Maryland", "MD"),
    ("Massachusetts", "MA"),
    ("Michigan", "MI"),
    ("Minnesota", "MN"),
    ("Mississippi", "MS"),
    ("Missouri", "MO"),
    ("Montana", "MT"),
    ("Nebraska", "NE"),
    ("Nevada", "NV"),
    ("New Hampshire", "NH"),
    ("New Jersey", "NJ"),
    ("New Mexico", "NM"),
    ("New York", "NY"),
    ("North Carolina", "NC"),
    ("North Dakota", "ND"),
    ("Northern Mariana Islands", "MP"),
    ("Ohio", "OH"),
    ("Oklahoma", "OK"),
    ("Oregon", "OR"),
    ("Pennsylvania", "PA"),
    ("Puerto Rico", "PR"),
    ("Rhode Island", "RI"),
    ("South Carolina", "SC"),
    ("South Dakota", "SD"),
    ("Tennessee", "TN"),
    ("Texas", "TX"),
    ("Utah", "UT"),
    ("Vermont", "VT"),
    ("Virgin Islands", "VI"),
    ("Virginia", "VA"),
    ("Washington", "WA"),
    ("West Virginia", "WV"),
    ("Wisconsin", "WI"),
    ("Wyoming", "WY"),
)

USA_STATE_NAME_TO_CODE = dict(USA_STATES)

USA_STATE_CODE_TO_NAME = dict((v, k) for k, v in USA_STATE_NAME_TO_CODE.items())



from zs_utils.api.base_api import BaseAPI


class AliexpressRussiaAPI(BaseAPI):
    production_api_url = "https://openapi.aliexpress.ru"
    http_method = "POST"

    def __init__(self, access_token: str, **kwargs):
        super().__init__(**kwargs)

        self.access_token = access_token

    @property
    def headers(self) -> dict:
        return {
            "x-auth-token": self.access_token,
            "Accept": "application/json",
            "Content-Type": "application/json",
        }



from zs_utils.api.base_api import BaseAPI


class WildberriesAPI(BaseAPI):
    production_api_url = "https://suppliers-api.wildberries.ru/"

    def __init__(self, api_token: str, **kwargs):
        super().__init__(**kwargs)

        self.api_token = api_token

    @property
    def headers(self) -> dict:
        return {"Content-Type": "application/json", "Authorization": self.api_token}

    def get_request_params(self, **kwargs) -> dict:
        request_params = {
            "url": self.build_url(kwargs),
            "headers": self.headers,
        }

        clean_params = self.get_clean_params(kwargs)
        clean_params.pop("files", None)

        if self.http_method == "GET":
            request_params["params"] = clean_params
        else:
            if self.array_payload:
                request_params["json"] = clean_params["array_payload"]
            else:
                request_params["json"] = clean_params

        return request_params



from model_utils import Choices

from django.conf import settings
from django.utils.translation import gettext_lazy as _


SUPPORT_TICKET_QUESTION_TYPES = getattr(
    settings,
    "SUPPORT_TICKET_QUESTION_TYPES",
    Choices(
        ("BASIC", _("Общий вопрос")),
        ("INTERFACE", _("Интерфейс")),
        ("TECHNICAL", _("Технический")),
        ("BILLING", _("Счет")),
    )
)

SUPPORT_TICKET_STATUSES = Choices(
    ("PENDING", _("Ожидает рассмотрения")),
    ("OPEN", _("Открыто")),
    ("CLOSED_BY_USER", _("Закрыто пользователем")),
    ("CLOSED_BY_MANAGER", _("Закрыто менеджером")),
    ("CLOSED_AUTO", _("Закрыто из-за отсутствия активности")),
)

SUPPORT_TICKET_ACTIVE_STATUSES = SUPPORT_TICKET_STATUSES.subset("PENDING", "OPEN")

SUPPORT_TICKET_ACTIVE_STATUSES_LIST = [item[0] for item in SUPPORT_TICKET_ACTIVE_STATUSES]

SUPPORT_TICKET_CLIENT_STATUSES = Choices(
    ("PENDING", _("В работе")),
    ("RESPONDED", _("Получен ответ")),
    ("CLOSED", _("Закрыто")),
)

SUPPORT_TICKET_MESSAGES = Choices(
    ("OPEN", _("Заявка взята в обработку оператором.")),
    ("CLOSED_AUTO", _("Заявка автоматически закрыта после длительного отсутствия активности.")),
    ("CLOSED_BY_USER", _("Заявка закрыта пользователем.")),
    ("CLOSED_BY_MANAGER", _("Заявка закрыта оператором.")),
    ("REOPEN", _("Заявка повторно открыта.")),
)

SUPPORT_TICKET_MESSAGE_SIGNATURE = getattr(settings, "SUPPORT_TICKET_MESSAGE_SIGNATURE", "")

MAX_OPEN_TICKET = getattr(settings, "MAX_OPEN_TICKET", 10)
MAX_TEXT_LENGTH = getattr(settings, "MAX_TEXT_LENGTH", 1500)
AUTO_CLOSE_TICKET_AFTER = getattr(settings, "AUTO_CLOSE_TICKET_AFTER", 2)



from zs_utils.api.ebay.base_api import EbayAPI


class GetPrivileges(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/account/resources/privilege/methods/getPrivileges
    """

    http_method = "GET"
    resource_method = "sell/account/v1/privilege"



from yookassa.domain.notification import WebhookNotification

from rest_framework import status
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.exceptions import ValidationError

from zs_utils.api.yookassa.services import BaseYooKassaService
from zs_utils.views import CustomAPIView


__all__ = [
    "BaseYooKassaWebhookView",
]


class BaseYooKassaWebhookView(CustomAPIView):
    """
    View для получения уведомлений от YooKassa.
    """

    IP_VALIDATION = False
    YOOKASSA_WEBHOOK_IP_WHITELIST = []
    YOOKASSA_SERVICE = BaseYooKassaService

    @staticmethod
    def validation(event_type: str, event_data: dict, metadata: dict):
        pass

    @property
    def request_ip(self) -> str:
        return None

    def get_user(self, user_id: str = None, payment_id: str = None):
        raise NotImplementedError("Необходимо определить метод получения пользователя.")

    def post(self, request: Request, *args, **kwargs):
        # IP validation
        if self.IP_VALIDATION and self.request_ip not in self.YOOKASSA_WEBHOOK_IP_WHITELIST:
            return Response(status=status.HTTP_403_FORBIDDEN)

        # Валидация ключей
        for key in ["event", "object"]:
            if not request.data.get(key):
                raise ValidationError({key: "Это поле обязательно."})

        # Разбор данных
        notification = WebhookNotification(request.data)
        event_type = notification.event
        event_data = dict(notification.object)
        metadata = event_data.get("metadata", {})

        self.validation(event_type=event_type, event_data=event_data, metadata=metadata)

        # Определяем пользователя
        user_id = metadata.get("user_id")
        payment_id = request.data["object"].get("payment_id", event_data.get("id"))
        event_data["payment_id"] = payment_id
        user = self.get_user(user_id=user_id, payment_id=payment_id)
        if not user:
            return Response({"detail": "Не удалось определить пользователя"}, status.HTTP_400_BAD_REQUEST)

        # Обработка данных
        self.YOOKASSA_SERVICE.process_webhook(user=user, event_type=event_type, event_data=event_data)

        return Response(status=status.HTTP_200_OK)



# Generated by Django 4.1.5 on 2023-02-23 12:23

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import uuid
import zs_utils.json_utils


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="APIRequestLog",
            fields=[
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                (
                    "id",
                    model_utils.fields.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                ("url", models.TextField(verbose_name="URL запроса")),
                (
                    "method",
                    models.CharField(
                        choices=[
                            ("GET", "GET"),
                            ("POST", "POST"),
                            ("DELETE", "DELETE"),
                            ("PUT", "PUT"),
                            ("PATCH", "PATCH"),
                        ],
                        max_length=6,
                        verbose_name="Метод запроса",
                    ),
                ),
                (
                    "params",
                    models.JSONField(
                        blank=True,
                        encoder=zs_utils.json_utils.CustomJSONEncoder,
                        null=True,
                        verbose_name="Параметры URL",
                    ),
                ),
                (
                    "request_headers",
                    models.JSONField(
                        blank=True,
                        encoder=zs_utils.json_utils.CustomJSONEncoder,
                        null=True,
                        verbose_name="Хедеры запроса",
                    ),
                ),
                (
                    "request_body",
                    models.JSONField(
                        blank=True,
                        encoder=zs_utils.json_utils.CustomJSONEncoder,
                        null=True,
                        verbose_name="Тело запроса",
                    ),
                ),
                ("response_time", models.IntegerField(verbose_name="Время ответа (миллисекунда)")),
                ("status_code", models.IntegerField(verbose_name="Код ответа")),
                (
                    "response_headers",
                    models.JSONField(
                        blank=True,
                        encoder=zs_utils.json_utils.CustomJSONEncoder,
                        null=True,
                        verbose_name="Хедеры ответа",
                    ),
                ),
                (
                    "response_body",
                    models.JSONField(
                        blank=True, encoder=zs_utils.json_utils.CustomJSONEncoder, null=True, verbose_name="Тело ответа"
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="api_request_logs",
                        related_query_name="api_request_log",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Пользователь",
                    ),
                ),
            ],
            options={
                "ordering": ["-created"],
            },
        ),
    ]



from zs_utils.api.aliexpress.base_api import AliexpressAPI


class GetLogisticsServiceListAPI(AliexpressAPI):
    resource_method = "aliexpress.logistics.redefining.listlogisticsservice"


class GetOnlineLogisticsServiceByOrderIdAPI(AliexpressAPI):
    resource_method = "aliexpress.logistics.redefining.getonlinelogisticsservicelistbyorderid"
    required_params = ["order_id"]
    allowed_params = ["goods_width", "goods_height", "goods_weight", "goods_length"]


class CreateWarehouseOrderAPI(AliexpressAPI):
    resource_method = "logistics.createwarehouseorder"
    required_params = [
        "address_d_t_os",
        "declare_product_d_t_os",
        "domestic_logistics_company_id",
        "domestic_tracking_no",
        "trade_order_from",
        "trade_order_id",
        "warehouse_carrier_service",
    ]
    allowed_params = [
        "domestic_logistics_company",
        "package_num",
        "undeliverable_decision",
        "invoice_number",
        "top_user_key",
    ]



from zs_utils.api.apiship.base_api import ApishipAPI


class ApishipGetOrderStatusAPI(ApishipAPI):
    resource_method = "orders/{order_id}/status"
    http_method = "GET"
    required_params = ["order_id"]


class ApishipGetOrderStatusByClientNumberAPI(ApishipAPI):
    resource_method = "orders/status"
    http_method = "GET"
    required_params = ["clientNumber"]


class ApishipGetOrderStatusHistoryAPI(ApishipAPI):
    resource_method = "orders/{order_id}/statusHistory"
    http_method = "GET"
    required_params = ["order_id"]
    allowed_params = ["offset", "limit"]


class ApishipGetOrderStatusHistoryByClientNumberAPI(ApishipAPI):
    resource_method = "orders/status/history"
    http_method = "GET"
    required_params = ["clientNumber"]


class ApishipGetOrderStatusListAPI(ApishipAPI):
    resource_method = "orders/statuses"
    http_method = "POST"
    required_params = ["orderIds"]


class ApishipGetOrdersWithChangedStatusAPI(ApishipAPI):
    resource_method = "orders/statuses/date/{date}"
    http_method = "GET"
    required_params = ["date"]


class ApishipGetOrderStatusHistoryListByDateAPI(ApishipAPI):
    resource_method = "orders/statuses/history/date/{date}"
    http_method = "GET"
    required_params = ["date"]
    allowed_params = ["offset", "limit"]


class ApishipGetOrdersStatusChangeByIntervalAPI(ApishipAPI):
    resource_method = "orders/statuses/interval"
    http_method = "GET"
    required_params = ["from", "to"]
    allowed_params = ["offset", "limit", "filter"]



from django_filters import filterset, filters

from zs_utils.support import constants, models


__all__ = [
    "SupportTicketFilter",
]


class SupportTicketFilter(filterset.FilterSet):
    active = filters.BooleanFilter(method="filter_active")
    client_status = filters.ChoiceFilter(
        choices=constants.SUPPORT_TICKET_CLIENT_STATUSES,
        method="filter_by_client_status",
    )
    ordering = filters.OrderingFilter(fields=["created"])

    class Meta:
        model = models.SupportTicket
        fields = [
            "user",
            "manager",
            "status",
            "question_type",
            "number",
        ]

    def get_responded_ticket_ids(self, queryset) -> list[str]:
        responded_ids = []
        for ticket in queryset.filter(status=constants.SUPPORT_TICKET_STATUSES.OPEN):
            if ticket.manager:
                last_message = ticket.last_message
                if last_message and (ticket.last_message.sender == ticket.manager):
                    responded_ids.append(ticket.id)
        return responded_ids

    def filter_by_client_status(self, queryset, name, value):
        if value == constants.SUPPORT_TICKET_CLIENT_STATUSES.PENDING:
            return queryset.filter(status__in=constants.SUPPORT_TICKET_ACTIVE_STATUSES_LIST).exclude(
                id__in=self.get_responded_ticket_ids(queryset=queryset)
            )
        if value == constants.SUPPORT_TICKET_CLIENT_STATUSES.RESPONDED:
            return queryset.filter(id__in=self.get_responded_ticket_ids(queryset=queryset))
        if value == constants.SUPPORT_TICKET_CLIENT_STATUSES.CLOSED:
            return queryset.exclude(status__in=constants.SUPPORT_TICKET_ACTIVE_STATUSES_LIST)

    def filter_active(self, queryset, name, value):
        filter_kwargs = {"status__in": constants.SUPPORT_TICKET_ACTIVE_STATUSES_LIST}
        if value:
            return queryset.filter(**filter_kwargs)
        else:
            return queryset.exclude(**filter_kwargs)



from rest_framework import serializers

from zs_utils.api import models


__all__ = [
    "APIRequestLogSerializer",
    "APIRequestLogListSerializer",
]


class APIRequestLogSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.APIRequestLog
        fields = [
            "id",
            "created",
            "user",
            "is_success",
            "url",
            "method",
            "params",
            "request_headers",
            "request_body",
            "status_code",
            "response_time",
            "response_headers",
            "response_body",
        ]


class APIRequestLogListSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.APIRequestLog
        fields = [
            "id",
            "created",
            "user",
            "is_success",
            "url",
            "method",
            "status_code",
            "response_time",
        ]



from zs_utils.api.base_api import BaseAPI


class ApishipAPI(BaseAPI):
    production_api_url = "https://api.apiship.ru/v1/"
    sandbox_api_url = "http://api.dev.apiship.ru/v1/"

    def __init__(self, access_token: str = None, **kwargs):
        super().__init__(**kwargs)

        self.access_token = access_token

    @property
    def headers(self) -> dict:
        header = {
            "Content-Type": "application/json",
        }
        if self.access_token:
            header["Authorization"] = self.access_token
        return header



from zs_utils.api.etsy.base_api import EtsyAPI


class FindAllShops(EtsyAPI):
    """
    https://www.etsy.com/openapi/developers#operation/findShops
    """

    http_method = "GET"
    resource_method = "shops"
    required_params = ["shop_name"]
    allowed_params = ["limit", "offset"]


class GetEtsyShopByUserId(EtsyAPI):
    """
    https://www.etsy.com/openapi/developers#operation/getShopByOwnerUserId
    """

    http_method = "GET"
    resource_method = "users/{user_id}/shops"
    required_params = ["user_id"]


class GetEtsyShop(EtsyAPI):
    """
    https://www.etsy.com/openapi/developers#operation/getShop
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}"
    required_params = ["shop_id"]


class UpdateEtsyShop(EtsyAPI):
    """
    https://www.etsy.com/openapi/developers#operation/updateShop
    """

    http_method = "PUT"
    resource_method = "shops/{shop_id}"
    required_params = ["shop_id"]
    allowed_params = [
        "title",
        "announcement",
        "sale_message",
        "digital_sale_message",
    ]



from zs_utils.api.aliexpress.base_api import AliexpressAPI


class GetDropshippingProductInfo(AliexpressAPI):
    resource_method = "aliexpress.postproduct.redefining.findaeproductbyidfordropshipper"
    allowed_params = ["product_id", "local_country", "local_language"]


class DropshippingPlaceOrderAPI(AliexpressAPI):
    resource_method = "aliexpress.trade.buy.placeorder"
    required_params = ["param_place_order_request4_open_api_d_t_o"]


class GetDropshippingLogisticInfoAPI(AliexpressAPI):
    resource_method = "aliexpress.logistics.buyer.freight.calculate"
    required_params = ["param_aeop_freight_calculate_for_buyer_d_t_o"]


class GetDropshippingOrderInfoAPI(AliexpressAPI):
    resource_method = "aliexpress.trade.ds.order.get"
    required_params = ["single_order_query"]


class GetDropshippingSimpleProductInfoAPI(AliexpressAPI):
    resource_method = "aliexpress.offer.ds.product.simplequery"
    allowed_params = ["product_id", "local_country", "local_language"]


class GetDropshippingTrackingInfoAPI(AliexpressAPI):
    resource_method = "aliexpress.logistics.ds.trackinginfo.query"
    required_params = ["logistics_no", "origin", "out_ref", "service_name", "to_area"]


class SyncDropshippingSalesDataAPI(AliexpressAPI):
    resource_method = "aliexpress.member.ds.orderdata.save"
    required_params = ["dser_collect_data"]



from zs_utils.api.etsy.base_api import EtsyAPI


class GetEtsyProduct(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getListingProduct
    """

    http_method = "GET"
    resource_method = "listings/{listing_id}/inventory/products/{product_id}"
    required_params = ["listing_id", "product_id"]


class GetEtsyOffering(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getListingOffering
    """

    http_method = "GET"
    resource_method = "listings/{listing_id}/products/{product_id}/offerings/{product_offering_id}"
    required_params = ["listing_id", "product_id", "product_offering_id"]


class GetEtsyListingProductList(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getListingInventory
    """

    http_method = "GET"
    resource_method = "listings/{listing_id}/inventory"
    required_params = ["listing_id"]


class UpdateEtsyListingInventory(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/updateListingInventory
    """

    http_method = "PUT"
    resource_method = "listings/{listing_id}/inventory"
    required_params = [
        "listing_id",
        "products",
    ]
    allowed_params = [
        "price_on_property",
        "quantity_on_property",
        "sku_on_property",
    ]


class GetEtsyListingList(EtsyAPI):
    """
    Endpoint to list Listings that belong to a Shop. Listings can be filtered using the 'state' param.

    Docs:
    https://www.etsy.com/openapi/developers#operation/getListings
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}/listings"
    required_params = ["shop_id"]
    allowed_params = ["state", "limit", "offset", "sort_on", "sort_order"]


class GetEtsySingleListing(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/developers/documentation/reference/listing#method_getlisting
    """

    http_method = "GET"
    resource_method = "listings/{listing_id}"
    required_params = ["listing_id"]


class GetEtsyListingAttributeList(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getListingProperties
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}/listings/{listing_id}/properties"
    required_params = ["shop_id", "listing_id"]


class GetEtsyListingAttribute(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getListingProperty
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}/listings/{listing_id}/properties/{property_id}"
    required_params = ["shop_id", "listing_id", "property_id"]


class CreateOrUpdateEtsyListingAttribute(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/updateListingProperty
    """

    http_method = "PUT"
    resource_method = "shops/{shop_id}/listings/{listing_id}/properties/{property_id}"
    required_params = [
        "shop_id",
        "listing_id",
        "property_id",
        "values",
    ]
    allowed_params = [
        "value_ids",
        "scale_id",
    ]


class DeleteEtsyListingAttribute(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/deleteListingProperty
    """

    http_method = "DELETE"
    resource_method = "shops/{shop_id}/listings/{listing_id}/properties/{property_id}"
    required_params = ["shop_id", "listing_id", "property_id"]


class DeleteEtsyListing(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/deleteListing
    """

    http_method = "DELETE"
    resource_method = "listings/{listing_id}"
    required_params = ["listing_id"]


class GetEtsyOrderListings(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getListingsByShopReceipt
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}/receipts/{receipt_id}/listings"
    required_params = ["receipt_id", "shop_id"]


class GetEtsyShopSectionListings(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getListingsByShopSectionId
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}/sections/{shop_section_id}/listings"
    required_params = ["shop_id", "shop_section_id"]
    allowed_params = ["limit", "offset", "sort_on", "sort_order"]


class CreateEtsySingleListing(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/createListing
    """

    http_method = "POST"
    resource_method = "shops/{shop_id}/listings"
    required_params = [
        "shop_id",
        "quantity",
        "title",
        "description",
        "price",
        "who_made",
        "when_made",
        "taxonomy_id",
        "shipping_profile_id",
    ]
    allowed_params = [
        "materials",
        "shop_section_id",
        "processing_min",
        "processing_max",
        "tags",
        "recipient",
        "occasion",
        "styles",
        "is_personalizable",
        "personalization_is_required",
        "personalization_char_count_max",
        "personalization_instructions",
        "production_partner_ids",
        "image_ids",
        "is_supply",
        "is_customizable",
        "is_taxable",
        "item_length",
        "item_width",
        "item_height",
        "item_dimensions_unit",
        "item_weight",
        "item_weight_unit",
        "type",
    ]


class UpdateEtsySingleListing(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/updateListing
    """

    http_method = "PUT"
    resource_method = "shops/{shop_id}/listings/{listing_id}"
    required_params = ["shop_id", "listing_id"]
    allowed_params = [
        "quantity",
        "price",
        "title",
        "description",
        "materials",
        "should_auto_renew",
        "shipping_profile_id",
        "shop_section_id",
        "is_taxable",
        "taxonomy_id",
        "tags",
        "who_made",
        "when_made",
        "featured_rank",
        "is_personalizable",
        "personalization_is_required",
        "personalization_char_count_max",
        "personalization_instructions",
        "state",
        "is_supply",
        "production_partner_ids",
        "type",
        "item_length",
        "item_width",
        "item_height",
        "item_dimensions_unit",
        "item_weight",
        "item_weight_unit",
    ]


class DeleteListingFile(EtsyAPI):
    """
    Docs: https://developers.etsy.com/documentation/reference/#operation/getListingsByShopSectionId
    """

    http_method = "DELETE"
    resource_method = "shops/{shop_id}/listings/{listing_id}/files/{listing_file_id}"
    required_params = ["shop_id", "listing_id", "listing_file_id"]


class GetListingFile(EtsyAPI):
    """
    Docs: https://developers.etsy.com/documentation/reference/#operation/getListingFile
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}/listings/{listing_id}/files/{listing_file_id}"
    required_params = ["shop_id", "listing_id", "listing_file_id"]


class GetAllListingFiles(EtsyAPI):
    """
    Docs: https://developers.etsy.com/documentation/reference/#operation/getAllListingFiles
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}/listings/{listing_id}/files"
    required_params = ["shop_id", "listing_id"]


class UploadListingFile(EtsyAPI):
    """
    Docs: https://developers.etsy.com/documentation/reference/#operation/uploadListingFile
    """

    http_method = "POST"
    resource_method = "shops/{shop_id}/listings/{listing_id}/files"
    required_params = ["shop_id", "listing_id"]
    allowed_params = [
        "listing_file_id",
        "file",
        "name",
        "rank",
    ]

    @property
    def headers(self) -> dict:
        # При наличии файла будет автоматом подставлен multipart - что нам и нужно
        headers = super().headers
        headers.pop("Content-Type")
        return headers



from zs_utils.api.etsy.base_api import EtsyAPI


class CreateEtsyShippingProfileUpgrade(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/createListingShippingProfileUpgrade
    """

    http_method = "POST"
    resource_method = "shops/{shop_id}/shipping-profiles/{shipping_profile_id}/upgrades"
    required_params = [
        "shop_id",
        "shipping_profile_id",
        "type",
        "upgrade_name",
        "price",
        "secondary_price",
    ]
    allowed_params = [
        "shipping_carrier_id",
        "mail_class",
        "min_delivery_days",
        "max_delivery_days",
    ]


class GetEtsyShippingProfileUpgradeList(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getListingShippingProfileUpgrades
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}/shipping-profiles/{shipping_profile_id}/upgrades"
    required_params = ["shop_id", "shipping_profile_id"]


class UpdateEtsyShippingProfileUpgrade(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/updateListingShippingProfileUpgrade
    """

    http_method = "PUT"
    resource_method = "shops/{shop_id}/shipping-profiles/{shipping_profile_id}/upgrades/{upgrade_id}"
    required_params = ["shop_id", "shipping_profile_id", "upgrade_id"]
    allowed_params = [
        "upgrade_name",
        "type",
        "price",
        "secondary_price",
        "shipping_carrier_id",
        "mail_class",
        "min_delivery_days",
        "max_delivery_days",
    ]


class DeleteEtsyShippingProfileUpgrade(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/updateListingShippingProfileUpgrade
    """

    http_method = "DELETE"
    resource_method = "shops/{shop_id}/shipping-profiles/{shipping_profile_id}/upgrades/{upgrade_id}"
    required_params = ["shop_id", "shipping_profile_id", "upgrade_id"]



from zs_utils.api.yandex_market.base_api import YandexMarketAPI


class SendYandexMarketShipmentBoxesAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-marketplace-cd/doc/dg/reference/put-campaigns-id-orders-id-delivery-shipments-id-boxes.html
    """

    description = "Передача информации о грузовых местах в заказе Yandex Market"
    http_method = "PUT"
    resource_method = "campaigns/{shop_id}/orders/{order_id}/delivery/shipments/{shipment_id}/boxes"
    required_params = [
        "shop_id",
        "order_id",
        "shipment_id",
        "boxes",
    ]


class GetYandexMarketTransferActAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-marketplace-cd/doc/dg/reference/get-campaigns-id-shipments-reception-transfer-act.html
    """

    description = "Получение акта приема-передачи сегодняшних заказов Yandex Market"
    http_method = "GET"
    resource_method = "campaigns/{shop_id}/shipments/reception-transfer-act"
    required_params = [
        "shop_id",
    ]


class GetYandexMarketRegions(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-regions.html
    """

    description = "Поиск региона"
    http_method = "GET"
    resource_method = "regions"
    required_params = [
        "name",
    ]


class GetYandexMarketDeliveryServicesAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-delivery-services.html
    """

    description = "Получение служб доставки Yandex Market"
    http_method = "GET"
    resource_method = "delivery/services"


class GetYandexMarketOrderListAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-campaigns-id-orders.html
    """

    description = "Получение списка заказов Yandex Market"
    http_method = "GET"
    resource_method = "campaigns/{shop_id}/orders"
    allowed_params = [
        "shop_id",
        "fromDate",
        "toDate",
        "status",
        "substatus",
        "page",
        "pageSize",
    ]


class GetYandexMarketOrderAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-campaigns-id-orders-id.html
    """

    description = "Получения заказа Yandex Market"
    http_method = "GET"
    resource_method = "campaigns/{shop_id}/orders/{order_id}"
    required_params = [
        "shop_id",
        "order_id",
    ]


class ChangeYandexMarketOrderStatusAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/put-campaigns-id-orders-id-status.html
    """

    description = "Изменение статуса заказа Yandex Market"
    http_method = "PUT"
    resource_method = "campaigns/{shop_id}/orders/{order_id}/status"
    required_params = [
        "shop_id",
        "order_id",
        "order",
    ]


class ChangeYandexMarketDigitalOrderStatusAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/troubleshooting/general.html#general__digital
    """

    description = "Изменение статуса цифрового заказа Yandex Market"
    http_method = "POST"
    resource_method = "campaigns/{shop_id}/orders/{order_id}/deliverDigitalGoods"
    required_params = [
        "shop_id",
        "order_id",
    ]


class GetYandexMarketShipmentLabelsAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-marketplace-cd/doc/dg/reference/get-campaigns-id-orders-id-delivery-labels.html
    """

    description = "Получение ярлыков‑наклеек на все грузовые места в заказе Yandex Market"
    http_method = "GET"
    resource_method = "campaigns/{shop_id}/orders/{order_id}/delivery/labels"
    required_params = [
        "shop_id",
        "order_id",
    ]


class SetYandexMarketShipmentTrackAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/post-campaigns-id-orders-id-delivery-track.html
    """

    description = "Передача трек-номера посылки"
    http_method = "POST"
    resource_method = "campaigns/{shop_id}/orders/{order_id}/delivery/track"
    required_params = [
        "shop_id",
        "order_id",
    ]



MarketplaceToHost = {
    "default": "ebay.com",
    "EBAY_US": "ebay.com",
    "EBAY_MOTORS_US": "ebay.com/motors",
    "EBAY_CA": "ebay.ca",
    "EBAY_GB": "ebay.co.uk",
    "EBAY_AU": "ebay.com.au",
    "EBAY_AT": "ebay.at",
    "EBAY_BE_FR": "befr.ebay.be",
    "EBAY_FR": "ebay.fr",
    "EBAY_DE": "ebay.de",
    "EBAY_IT": "ebay.it",
    "EBAY_BE_NL": "benl.ebay.be",
    "EBAY_NL": "ebay.nl",
    "EBAY_ES": "ebay.es",
    "EBAY_CH": "ebay.ch",
    "EBAY_TW": "ebay.com/tw",
    "EBAY_CZ": "ebay.com/sch/Czech-Republic",
    "EBAY_DK": "ebay.com/sch/Denmark",
    "EBAY_FI": "ebay.com/sch/Finland",
    "EBAY_GR": "ebay.com/sch/Greece",
    "EBAY_HK": "ebay.com.hk",
    "EBAY_HU": "ebay.com/sch/Hungary",
    "EBAY_IN": "ebay.in",
    "EBAY_ID": "id.ebay.com",
    "EBAY_IE": "ebay.ie",
    "EBAY_IL": "ebay.com/sch/Israel",
    "EBAY_MY": "ebay.com/my",
    "EBAY_NZ": "ebay.com/sch/New-Zealand",
    "EBAY_NO": "ebay.com/sch/Norway",
    "EBAY_PH": "ebay.ph",
    "EBAY_PL": "ebay.pl",
    "EBAY_PT": "ebay.com/sch/Portugal",
    "EBAY_PR": "ebay.com/sch/Puerto-Rico",
    "EBAY_RU": "ebay.com/sch/Russia",
    "EBAY_SG": "ebay.com/sg",
    "EBAY_ZA": "ebay.com/sch/South-Africa",
    "EBAY_SE": "ebay.com/sch/Sweden",
    "EBAY_TH": "export.ebay.co.th",
    "EBAY_VN": "ebay.vn",
    "EBAY_CN": "ebay.com/sch/China",
    "EBAY_PE": "ebay.com/sch/Peru",
    "EBAY_CA_FR": "cafr.ebay.ca",
    "EBAY_JP": "ebay.co.jp",
}



from datetime import datetime

from .abstract import AbstractModel, XMLElement
from .shop import Shop

DATE_FORMAT = "%Y-%m-%d %H:%M:%S"


class Feed(AbstractModel):
    """
    YML Feed model.

    Docs:
    https://yandex.ru/support/partnermarket/export/yml.html
    """

    __slots__ = ["shop", "_date"]

    def __init__(self, shop: Shop, date: datetime.date = None):
        self.shop = shop
        self.date = date

    @property
    def date(self) -> datetime:
        return datetime.strptime(self._date, DATE_FORMAT)

    @date.setter
    def date(self, dt):
        dt = self._is_valid_datetime(dt, DATE_FORMAT, "date", True)
        if dt is None:
            dt = datetime.now().strftime(DATE_FORMAT)
        self._date = dt

    def create_dict(self, **kwargs) -> dict:
        return dict(
            shop=self.shop.to_dict(),
            date=self.date,
        )

    def create_xml(self, **kwargs) -> XMLElement:
        feed_el = XMLElement("yml_catalog", {"date": self._date})
        self.shop.to_xml(feed_el)
        return feed_el

    @staticmethod
    def from_xml(el: XMLElement) -> "Feed":
        shop = Shop.from_xml(el[0])
        date = el.attrib.get("date")
        return Feed(shop, date=date)



from zs_utils.api.aliexpress_russia.base_api import AliexpressRussiaAPI


class GetAliexpressMessageListAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/manage-instant-messages#heading-poluchit-soobscheniya
    """

    http_method = "GET"
    resource_method = "/api/v2/seller/chat-messages"
    allowed_params = [
        "from_message_id",
        "limit",  # max 50
    ]


class SendAliexpressMessageAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/manage-instant-messages#heading-otpravit-soobschenie
    """

    resource_method = "/api/v2/seller/new-messages"
    required_params = [
        "to_user_login_id",
        "created_at",
        "type",
        "payload",
    ]


class MarkAsReadAliexpressMessageAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/manage-instant-messages#heading-prochitat-soobschenie
    """

    resource_method = "/api/v2/seller/read-messages"
    required_params = [
        "to_user_login_id",
        "created_at",
    ]



from zs_utils.api.shopify.base_api import ShopifyAPI


class GetShopifyProductListAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/product#get-products
    """

    http_method = "GET"
    resource_method = "products.json"
    allowed_params = [
        "collection_id",
        "created_at_max",
        "created_at_min",
        "fields",
        "handle",
        "ids",
        "limit",  # <= 250
        "presentment_currencies",
        "product_type",
        "published_at_max",
        "published_at_min",
        "published_status",
        "since_id",
        "status",
        "title",
        "updated_at_max",
        "updated_at_min",
        "vendor",
        "page_info",
    ]


class GetShopifyProductCountAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/product#get-products-count
    """

    http_method = "GET"
    resource_method = "products/count.json"
    allowed_params = [
        "collection_id",
        "created_at_max",
        "created_at_min",
        "product_type",
        "published_at_max",
        "published_at_min",
        "published_status",
        "updated_at_max",
        "updated_at_min",
        "vendor",
    ]


class GetShopifyProductAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/product#get-products-product-id
    """

    http_method = "GET"
    resource_method = "products/{product_id}.json"
    required_params = ["product_id"]
    allowed_params = ["fields"]


class DeleteShopifyProductAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/product#delete-products-product-id
    """

    http_method = "DELETE"
    resource_method = "products/{product_id}.json"
    required_params = ["product_id"]


class UpdateShopifyProductAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/product#put-products-product-id
    """

    http_method = "PUT"
    resource_method = "products/{product_id}.json"
    payload_key = "product"
    required_params = ["product_id"]
    allowed_params = [
        "title",
        "status",
        "tags",
        "images",
        "variants",
        "metafields_global_title_tag",
        "metafields_global_description_tag",
        "published",
        "metafields",
    ]


class CreateShopifyProductAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/product#post-products
    """

    http_method = "POST"
    resource_method = "products.json"
    payload_key = "product"
    required_params = [
        "title",
    ]
    allowed_params = [
        "body_html",
        "vendor",
        "product_type",
        "tags",
        "status",
        "variants",
        "options",
        "handle",
        "images",
        "template_suffix",
        "metafields_global_title_tag",
        "metafields_global_description_tag",
        "metafields",
    ]



# Generated by Django 4.1.5 on 2023-02-23 12:23

from django.db import migrations, models
import model_utils.fields
import uuid


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="AmocrmApp",
            fields=[
                (
                    "id",
                    model_utils.fields.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                ("client_id", models.TextField(verbose_name="ID приложения")),
                ("client_secret", models.TextField(verbose_name="Секретный ключ приложения")),
                ("redirect_uri", models.TextField(verbose_name="Redirect URI приложения")),
                ("name", models.CharField(max_length=64, verbose_name="Локальный идентификатор")),
                ("access_token", models.TextField(null=True, verbose_name="Токен доступа")),
                ("access_token_expiry", models.DateTimeField(null=True, verbose_name="Дата истечения токена доступа")),
                ("refresh_token", models.TextField(null=True, verbose_name="Токен для обновления токена доступа")),
                (
                    "refresh_token_expiry",
                    models.DateTimeField(null=True, verbose_name="Дата истечения токена обновления"),
                ),
                ("is_default", models.BooleanField(default=False, verbose_name="Приложение по умолчанию")),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



import uuid
import os


__all__ = [
    "get_support_file_upload_path",
]


def get_unique_filename(filename: str):
    extension = os.path.splitext(filename)[1]
    return f"{str(uuid.uuid4())}{extension}"


def get_file_upload_path(
    filename: str,
    filename_prefix: str = None,
    dir_name: str = None,
    unique: bool = True,
):
    if unique:
        path = get_unique_filename(filename)
    else:
        path = filename

    if filename_prefix:
        path = filename_prefix + "_" + path

    if dir_name:
        path = dir_name + "/" + path

    return path


def get_support_file_upload_path(instance, filename: str) -> str:
    return get_file_upload_path(filename=filename, dir_name="support", unique=False)



from zs_utils.api.amocrm.base_api import AmocrmAPI


class ListAmocrmLeadsAPI(AmocrmAPI):
    """
    Docs: https://www.amocrm.ru/developers/content/crm_platform/leads-api#leads-list
    """

    http_method = "GET"
    resource_method = "api/v4/leads"
    required_params = []
    allowed_params = [
        "with",
        "page",
        "limit",
        "query",
        "filter",
        "order",
    ]


class GetAmocrmLeadAPI(AmocrmAPI):
    """
    Docs: https://www.amocrm.ru/developers/content/crm_platform/leads-api#lead-detail
    """

    http_method = "GET"
    resource_method = "api/v4/leads/{lead_id}"
    required_params = ["lead_id"]
    allowed_params = ["with"]


class CreateAmocrmLeadAPI(AmocrmAPI):
    """
    Docs: https://www.amocrm.ru/developers/content/crm_platform/leads-api#leads-add
    """

    http_method = "POST"
    resource_method = "api/v4/leads"
    array_payload = True


class UpdateAmocrmLeadAPI(AmocrmAPI):
    """
    Docs: https://www.amocrm.ru/developers/content/crm_platform/leads-api#leads-edit
    """

    http_method = "PATCH"
    resource_method = "api/v4/leads"
    array_payload = True


class ListAmocrmContactsAPI(AmocrmAPI):
    """
    Docs: https://www.amocrm.ru/developers/content/crm_platform/contacts-api#contacts-list
    """

    http_method = "GET"
    resource_method = "api/v4/contacts"
    required_params = []
    allowed_params = [
        "with",
        "page",
        "limit",
        "query",
        "filter",
        "order",
    ]


class CreateAmocrmContactAPI(AmocrmAPI):
    """
    Docs: https://www.amocrm.ru/developers/content/crm_platform/contacts-api#contacts-add
    """

    http_method = "POST"
    resource_method = "api/v4/contacts"
    array_payload = True


class CreateAmocrmTaskAPI(AmocrmAPI):
    """
    Docs: https://www.amocrm.ru/developers/content/crm_platform/tasks-api#tasks-add
    """

    http_method = "POST"
    resource_method = "api/v4/tasks"
    array_payload = True



from typing import List

from zs_utils.yandex_market_language.models import abstract


class Gift(abstract.AbstractModel):
    """
    Gift model.

    Docs:
    https://yandex.ru/support/partnermarket/elements/promo-gift.html
    """

    __slots__ = ["id", "name", "pictures"]

    def __init__(
        self,
        id: str,
        name: str,
        pictures: List[str] = None,
    ):
        super().__init__()
        self.id = id
        self.name = name
        self.pictures = pictures

    def create_dict(self, **kwargs) -> dict:
        return dict(id=self.id, name=self.name, pictures=self.pictures)

    def create_xml(self, **kwargs) -> "abstract.XMLElement":
        gift_el = abstract.XMLElement("gift", {"id": self.id})
        name_el = abstract.XMLSubElement(gift_el, "name")
        name_el.text = self.name

        # Add pictures
        if self.pictures:
            for url in self.pictures:
                p_el = abstract.XMLSubElement(gift_el, "picture")
                p_el.text = url

        return gift_el

    @staticmethod
    def from_xml(gift_el: abstract.XMLElement) -> "Gift":
        kwargs = {"pictures": []}

        # Parse element
        for el in gift_el:
            if el.tag == "name":
                kwargs["name"] = el.text
            elif el.tag == "picture":
                kwargs["pictures"].append(el.text)

        return Gift(id=gift_el.attrib.get("id"), **kwargs)



from zs_utils.api.base_api import BaseAPI


__all__ = [
    "YandexMarketAPI",
]


class YandexMarketAPI(BaseAPI):
    production_api_url = "https://api.partner.market.yandex.ru/v2/"

    def __init__(self, access_token: str, api_key: str, **kwargs):
        super().__init__(**kwargs)

        self.api_key = api_key
        self.access_token = access_token

    @property
    def headers(self) -> dict:
        return {
            "Authorization": f"OAuth oauth_token={self.access_token}, oauth_client_id={self.api_key}",
            "Content-Type": "application/json",
            "Accept": "application/json",
            "Accept-Charset": "utf-8",
            "Accept-Encoding": "application/gzip",
        }

    def build_url(self, params: dict) -> str:
        return super().build_url(params) + ".json"



#!/usr/bin/env python
# coding=utf-8


class UnConsistentUnitsError(ValueError):
    def __init__(self, *args):
        if args:
            self.value = "Units %s are not of the same dimension !" % str(args)
        else:
            self.value = "Units are not of the same dimension !"

    def __str__(self):
        return repr(self.value)


class UnitDoesntExistError(ValueError):
    def __init__(self, *args):
        if args:
            self.value = "Unit %s doesn't exist !" % str(args)
        else:
            self.value = "Unit doesn't exist !"

    def __str__(self):
        return repr(self.value)



from zs_utils.api.shopify.base_api import ShopifyAPI


class CreateShopifyWebhookAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/webhook#post-webhooks
    """

    http_method = "POST"
    resource_method = "webhooks.json"
    payload_key = "webhook"
    required_params = ["address", "topic"]
    allowed_params = ["format", "fields"]


class DeleteShopifyWebhookAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/webhook#delete-webhooks-webhook-id
    """

    http_method = "DELETE"
    resource_method = "webhooks/{webhook_id}.json"
    required_params = ["webhook_id"]


class GetShopifyWebhookListAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/webhook#get-webhooks
    """

    http_method = "GET"
    resource_method = "webhooks.json"
    allowed_params = [
        "address",
        "created_at_max",
        "created_at_min",
        "fields",
        "limit",  # <= 250 default 50
        "since_id",
        "topic",
        "updated_at_max",
        "updated_at_min",
        "page_info",
    ]


class UpdateShopifyWebhookAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/webhook#put-webhooks-webhook-id
    """

    http_method = "PUT"
    resource_method = "webhooks/{webhook_id}.json"
    payload_key = "webhook"
    required_params = ["webhook_id", "address"]



from zs_utils.api.yookassa import constants, core
from zs_utils.data.enums import CURRENCIES
from zs_utils.exceptions import CustomException
from zs_utils.user.models import AbstractZonesmartUser


__all__ = [
    "YooKassaException",
    "BaseYooKassaService",
]


class YooKassaException(CustomException):
    pass


class BaseYooKassaService:
    YOOKASSA_CREDENTIALS = None
    YOOKASSA_TEST_MODE = None

    @staticmethod
    def validate_amount(amount: float) -> None:
        """
        Валидация суммы amount (должна быть положительной)
        """
        if amount <= 0:
            raise YooKassaException("Значение 'amount' должно быть положительным.")

    @classmethod
    def convert_currency(cls, amount: float, from_currency: str) -> float:
        # TODO?
        # Always into CURRENCIES.RUB
        return amount

    @classmethod
    def get_receipt_template(
        cls,
        product_description: str,
        product_price: float,
        product_price_currency: CURRENCIES,
        user: AbstractZonesmartUser = None,
        inn: str = None,
        email: str = None,
        phone: str = None,
    ) -> dict:
        if user:
            email = user.email
            phone = getattr(user, "phone", None)
            organization = getattr(user, "user_organization", None)
            if organization:
                inn = organization.itn
            else:
                inn = None

        if product_price_currency != CURRENCIES.RUB:
            product_price = cls.convert_currency(amount=product_price, from_currency=product_price_currency)

        data = {
            "customer": {
                "inn": inn,
                "email": email,
                "phone": phone,
            },
            "items": [
                {
                    "description": product_description,
                    "quantity": 1,
                    "amount": {"value": product_price, "currency": "RUB"},
                    "vat_code": constants.YOOKASSA_VAT_CODES.n1,
                    "payment_subject": constants.YOOKASSA_PAYMENT_SUBJECTS.service,
                    "payment_mode": constants.YOOKASSA_PAYMENT_MODES.full_prepayment,
                    "agent_type": constants.YOOKASSA_AGENT_TYPES.payment_agent,
                },
            ],
            "tax_system_code": constants.YOOKASSA_TAX_SYSTEM_CODES.n3,
        }
        return data

    @classmethod
    def remote_setup_card(cls, user: AbstractZonesmartUser, return_url: str, extra_metadata: dict = None) -> tuple:
        """
        Создание платежа для подключения карты
        """
        call_params = {
            "user": user,
            "amount": 1,
            "description": "Подключение банковской карты.",
            "purpose": constants.YOOKASSA_PURPOSE.CARD_SETUP,
            "return_url": return_url,
            "extra_metadata": extra_metadata,
        }
        return cls.remote_create_payment(**call_params)

    @classmethod
    def remote_create_payment(
        cls,
        user: AbstractZonesmartUser,
        amount: float,
        return_url: str,
        purpose: str,
        description: str,
        payment_method_id: str = None,
        extra_metadata: dict = None,
    ) -> tuple:
        """
        Создание платежа через YooKassa
        """

        cls.validate_amount(amount)

        if not extra_metadata:
            extra_metadata = {}
        extra_metadata["purpose"] = purpose

        call_params = {
            "payment_method_id": payment_method_id,
            "amount": amount,
            "currency": "RUB",
            "description": description,
            "extra_metadata": extra_metadata,
            "return_url": return_url,
        }
        return core.RemoteCreateYooKassaPayment(
            credentials=cls.YOOKASSA_CREDENTIALS, test_mode=cls.YOOKASSA_TEST_MODE, user=user, propagate_exception=True
        )(**call_params)

    @classmethod
    def remote_create_refund(cls, user: AbstractZonesmartUser, payment_id: str, amount: float) -> tuple:
        """
        Запрос на возврат средств через YooKassa
        """

        cls.validate_amount(amount)

        return core.RemoteCreateYooKassaRefund(
            credentials=cls.YOOKASSA_CREDENTIALS, test_mode=cls.YOOKASSA_TEST_MODE, user=user, propagate_exception=True
        )(payment_id=payment_id, amount=amount)

    # Process webhook events
    @classmethod
    def process_webhook(cls, user: AbstractZonesmartUser, event_type: str, event_data: dict) -> None:
        """
        Обработка вебхук-уведомления от YooKassa
        """
        is_card_setup = False
        if event_data.get("metadata"):
            is_card_setup = event_data["metadata"].get("purpose") == constants.YOOKASSA_PURPOSE.CARD_SETUP

        if is_card_setup and event_type == "payment.canceled":
            action = cls.card_setup_canceled
        elif is_card_setup and event_type == "payment.succeeded":
            action = cls.card_setup_succeeded
        elif event_type == "payment.canceled":
            action = cls.payment_canceled
        elif event_type == "payment.succeeded":
            action = cls.payment_succeeded
        elif event_type == "refund.succeeded":
            action = cls.payment_refunded
        else:
            raise YooKassaException("Для данного события обработчик не найден.")

        action(user=user, raw_data=event_data)

    @classmethod
    def card_setup_succeeded(cls, user: AbstractZonesmartUser, raw_data: dict):
        return cls.create_or_update_payment_instance(user=user, raw_data=raw_data)

    @classmethod
    def payment_canceled(cls, user: AbstractZonesmartUser, raw_data: dict):
        return cls.create_or_update_payment_instance(user=user, raw_data=raw_data)

    @classmethod
    def payment_succeeded(cls, user: AbstractZonesmartUser, raw_data: dict):
        return cls.create_or_update_payment_instance(user=user, raw_data=raw_data)

    @classmethod
    def card_setup_canceled(cls, user: AbstractZonesmartUser, raw_data: dict):
        raise NotImplementedError("Необходимо определить метод обработки неудачной попытки подключить карту.")

    @classmethod
    def create_or_update_card_instance(cls, user: AbstractZonesmartUser, raw_data: dict):
        raise NotImplementedError("Необходимо определить метод создания и обновления банковской карты.")

    @classmethod
    def create_or_update_payment_instance(cls, user: AbstractZonesmartUser, raw_data: dict):
        raise NotImplementedError("Необходимо определить метод создания и обновления объекта платежа.")

    @classmethod
    def payment_refunded(cls, user: AbstractZonesmartUser, raw_data: dict):
        raise NotImplementedError("Необходимо определить метод обработки удачного возврата платежа.")



from zs_utils.api.insales.base_api import InsalesAPI


class GetInsalesCategoryList(InsalesAPI):
    """
    https://api.insales.ru/#category
    """

    resource_method = "categories.json"
    http_method = "GET"



from storages.backends.s3boto3 import S3Boto3Storage
from botocore.config import Config
from botocore.vendored.requests.exceptions import ConnectTimeout

from django.conf import settings
from django.utils.translation import gettext as _

from zs_utils.exceptions import CustomException


class S3Boto3StorageException(CustomException):
    status_code = 500


class CustomS3Boto3Storage(S3Boto3Storage):
    location = settings.AWS_MEDIA_ROOT
    default_acl = "public-read"
    config = Config(
        connect_timeout=settings.AWS_CONNECT_TIMEOUT,
        read_timeout=settings.AWS_READ_TIMEOUT,
        retries={
            "max_attempts": settings.AWS_CONNECT_MAX_RETRY_ATTEMPTS,
        },
    )

    def save(self, *args, **kwargs):
        try:
            return super().save(*args, **kwargs)
        except ConnectTimeout:
            raise S3Boto3StorageException(message=_("Проблемы с доступом к файловому хранилищу."))



from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from zs_utils.support import models


__all__ = [
    "CommonSupportTicketSerializer",
    "CommonCreateSupportTicketSerializer",
    "CommonSupportTicketMessageSerializer",
    "CommonCreateSupportTicketMessageSerializer",
    "CommonSupportTicketMessageFileSerializer",
]


class CommonSupportTicketSerializer(serializers.ModelSerializer):
    user = None
    manager = None

    class Meta:
        model = models.SupportTicket
        fields = [
            "id",
            "number",
            "created",
            "user",
            "manager",
            "status",
            "question_type",
            "subject",
            "unread_messages",
        ]


class CommonCreateSupportTicketSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.SupportTicket
        fields = [
            "question_type",
            "subject",
        ]


class CommonSupportTicketMessageSerializer(serializers.ModelSerializer):
    sender = None
    files = None

    class Meta:
        model = models.SupportTicketMessage
        fields = [
            "id",
            "created",
            "ticket",
            "sender",
            "text",
            "files",
            "is_system",
            "is_viewed",
        ]


class CommonCreateSupportTicketMessageSerializer(serializers.Serializer):
    text = serializers.CharField(required=False, allow_null=True)
    files = None

    def to_internal_value(self, data):
        data = super().to_internal_value(data)

        if not (data.get("text") or data.get("files")):
            raise ValidationError({"text": "Обязательное поле, если не задано поле 'files'."})

        return data


class CommonSupportTicketMessageFileSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.SupportTicketMessageFile
        fields = [
            "id",
            "ticket_message",
            "file",
        ]



from zs_utils.api.aliexpress_russia.base_api import AliexpressRussiaAPI


class GetAliexpressOrderListAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/local-get-orders-list
    """

    resource_method = "/seller-api/v1/order/get-order-list"
    required_params = ["page_size", "page"]
    allowed_params = [
        "date_start",
        "date_end",
        "order_statuses",
        "payment_statuses",
        "delivery_statuses",
        "antifraud_statuses",
        "order_ids",
        "sorting_order",
        "sorting_field",
        "tracking_numbers",
        "update_at_from",
        "update_at_to",
        "shipping_day_from",
        "shipping_day_to",
        "trade_order_info",
    ]


class SetAliexpressOrderInTransitAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/change-status#heading-prisvoit-zakazu-status-otpravlen
    """

    resource_method = "/api/v1/offline-ship/to-in-transit"
    required_params = [
        "trade_order_id",
        "tracking_number",
        "provider_name",
    ]
    allowed_params = [
        "tracking_url",
        "support_phone_number",
    ]


class SetAliexpressOrderReadyForPickupAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/change-status#heading-prisvoit-zakazu-status-ozhidaet-polucheniya
    """

    resource_method = "/api/v1/offline-ship/to-ready-for-pickup"
    required_params = ["trade_order_id"]


class SetAliexpressOrderDeliveredAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/change-status#heading-prisvoit-zakazu-status-dostavlen
    """

    resource_method = "/api/v1/offline-ship/to-delivered"
    required_params = ["trade_order_id"]



from html2text import html2text
from bs4 import BeautifulSoup


def is_html(text: str) -> bool:
    return bool(BeautifulSoup(text, "html.parser").find())


def html_to_plain_text(text: str, check_if_not_html: bool = True) -> str:
    if check_if_not_html or is_html(text=text):
        return html2text(html=text)
    return text



from zs_utils.api.wildberries.base_api import WildberriesAPI


class GetWildberriesCategoryList(WildberriesAPI):
    """
    Docs: https://openapi.wb.ru/content/api/ru/#tag/Konfigurator/paths/~1content~1v2~1object~1all/get
    """

    http_method = "GET"
    resource_method = "content/v2/object/all"
    allowed_params = ["name", "limit", "offset", "parentID"]


class GetWildberriesCategoryParentList(WildberriesAPI):
    """
    Docs: https://openapi.wb.ru/content/api/ru/#tag/Konfigurator/paths/~1content~1v2~1object~1parent~1all/get
    """

    http_method = "GET"
    resource_method = "content/v2/object/parent/all"


class GetWildberriesCategoryAttributes(WildberriesAPI):
    """
    Docs: https://openapi.wb.ru/content/api/ru/#tag/Konfigurator/paths/~1content~1v2~1object~1charcs~1%7BsubjectId%7D/get
    """

    http_method = "GET"
    resource_method = "content/v2/object/charcs/{subjectId}"
    required_params = ["subjectId"]


class GetWildberriesAttributeValues(WildberriesAPI):
    """
    Docs: https://openapi.wildberries.ru/#tag/Kontent-Konfigurator/paths/~1content~1v1~1directory~1colors/get
    Значения dictionary_name:
    - colors (Цвет)
    - kinds (Пол)
    - countries (Страна производства)
    - seasons (Сезон)
    - tnved (ТНВЭД)
    """

    http_method = "GET"
    resource_method = "content/v2/directory/{dictionary_name}"
    required_params = []
    allowed_params = [
        "search",  # только tnved
        "subjectID",  # только tnved
    ]



from setuptools import setup, find_packages


def parse_requirements() -> list[str]:
    reqs = []
    with open("requirements.txt") as f:
        for line in f.read().splitlines():
            line = line.split("#")[0].strip()
            if line:
                reqs.append(line)
    return reqs


setup(
    name="zonesmart-utils-fork",
    version="0.5.9",
    author="Zonesmart",
    author_email="e.beliakov@dev.kokoc.com",
    packages=find_packages(include=["zs_utils", "zs_utils.*"]),
    install_requires=parse_requirements(),
)



from requests import Session


__all__ = [
    "SessionWithResponseKeeper",
    "response_keeper",
]


# Кастомный класс, который вешает декоратор на метод self.send
class SessionWithResponseKeeper(Session):
    def __init__(self):
        super().__init__()
        self.send = response_keeper(self.send)


# Сохраняет в себя результат вызова (мы используем для хранения response)
def response_keeper(func):
    def wrapper(*args, **kwargs):
        raw_response = func(*args, **kwargs)
        response_keeper.raw_response = raw_response
        return raw_response

    return wrapper



from zs_utils.api.apiship.base_api import ApishipAPI


class ApishipGetRatesAPI(ApishipAPI):
    resource_method = "calculator"
    http_method = "POST"
    required_params = [
        "from",
        "to",
        "weight",
        "width",
        "height",
        "length",
    ]
    allowed_params = [
        "assessedCost",
        "pickupDate",
        "pickupTypes",
        "deliveryTypes",
        "codCost",
        "includeFees",
        "providerKeys",
        "timeout",
        "extraParams",
    ]



from zs_utils.api.base_api import BaseAPI


class FedexAPI(BaseAPI):
    production_api_url = "https://apis.fedex.com"
    sandbox_api_url = "https://apis-sandbox.fedex.com"

    def __init__(self, access_token: str, **kwargs):
        super().__init__(**kwargs)

        self.access_token = access_token

    @property
    def headers(self) -> dict:
        return {
            "Content-Type": "application/json",
            "X-locale": "en_US",
            "Authorization": f"Bearer {self.access_token}",
        }



from zs_utils.api.ebay.base_api import EbayAPI


class CreateOffer(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/createOffer
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/offer"


class UpdateOffer(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/updateOffer
    """

    http_method = "PUT"
    resource_method = "sell/inventory/v1/offer/{offerId}"
    required_params = ["offerId"]


class GetOffers(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/getOffers
    """

    http_method = "GET"
    resource_method = "sell/inventory/v1/offer"
    required_params = ["sku"]
    allowed_params = ["marketplace_id", "offset", "limit"]


class GetOffer(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/getOffer
    """

    http_method = "GET"
    resource_method = "sell/inventory/v1/offer/{offerId}"
    required_params = ["offerId"]


class DeleteOffer(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/deleteOffer
    """

    http_method = "DELETE"
    resource_method = "sell/inventory/v1/offer/{offerId}"
    required_params = ["offerId"]


class PublishOffer(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/publishOffer
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/offer/{offerId}/publish"
    required_params = ["offerId"]


class WithdrawOffer(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/withdrawOffer
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/offer/{offerId}/withdraw"
    required_params = ["offerId"]


class GetListingFees(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/getListingFees
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/offer/get_listing_fees"


class BulkCreateOffer(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/bulkCreateOffer
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/bulk_create_offer"


class BulkPublishOffer(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/bulkPublishOffer
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/bulk_publish_offer"



from zs_utils.api.ozon.base_api import OzonAPI


class OzonCreateActAPI(OzonAPI):
    resource_method = "v2/posting/fbs/act/create"
    required_params = ["delivery_method_id"]
    allowed_params = ["containers_count"]


class OzonCheckActStatusAPI(OzonAPI):
    resource_method = "v2/posting/fbs/act/check-status"
    required_params = ["id"]


class OzonGetActAPI(OzonAPI):
    resource_method = "v2/posting/fbs/act/get-pdf"
    required_params = ["id"]


class OzonGetContainerLabelsAPI(OzonAPI):
    resource_method = "v2/posting/fbs/act/get-container-labels"
    required_params = ["id"]



import datetime
import pytils

from django.utils import timezone


def now():
    return timezone.now()


def current_date():
    return now().date()


def days_ago_to_dates(days_ago: int, discard_time: bool = False):
    datetime_now = now()
    dates = {
        "date_before": datetime_now,
        "date_after": datetime_now - timezone.timedelta(days=int(days_ago)),
    }

    if discard_time:
        for key, value in dates.items():
            dates[key] = value.date()

    return dates


def from_datetime_str(value: str):
    if value.endswith("Z"):
        value = value[:-1] + "+00:00"
    return timezone.datetime.fromisoformat(value)


def datetime_str_to_date_str(value: str):
    return str(from_datetime_str(value).date())


def datetime_to_tsz(value) -> float:
    return timezone.datetime.timestamp(value)


def tsz_to_datetime(tsz, to_str=False, isoformat=False):
    dtm = timezone.make_aware(timezone.datetime.utcfromtimestamp(tsz), timezone=datetime.timezone.utc)
    if to_str:
        if isoformat:
            dtm = dtm.isoformat()
        else:
            dtm = dtm.strftime("%Y-%m-%d %H:%M:%S")
    return dtm


def datetime_to_words(value: timezone.timedelta, lang: str):
    seconds = int(value.total_seconds())
    periods = [
        ({"en": "year", "ru": ("год", "года", "лет")}, 60 * 60 * 24 * 365),
        ({"en": "month", "ru": ("месяц", "месяца", "месяцев")}, 60 * 60 * 24 * 30),
        ({"en": "day", "ru": ("день", "дня", "дней")}, 60 * 60 * 24),
        ({"en": "hour", "ru": ("час", "часа", "часов")}, 60 * 60),
        ({"en": "minute", "ru": ("минута", "минуты", "минут")}, 60),
        ({"en": "second", "ru": ("секунда", "секунды", "секунд")}, 1),
    ]

    strings = []
    for period_names, period_seconds in periods:
        if seconds >= period_seconds:
            period_value, seconds = divmod(seconds, period_seconds)
            if lang == "ru":
                strings.append(pytils.numeral.get_plural(amount=period_value, variants=period_names[lang]))
            elif lang == "en":
                has_s = "s" if period_value > 1 else ""
                strings.append("%s %s%s" % (period_value, period_names[lang], has_s))
            else:
                raise ValueError("Недопустимый язык 'lang'.")

    return ", ".join(strings)



from zs_utils.api.ebay.base_api import EbayTradingAPI


__all__ = [
    "GetAPILimits",
]


class GetAPILimits(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/devzone/xml/docs/reference/ebay/GetApiAccessRules.html
    """

    method_name = "GetApiAccessRules"

    def get_params(self, **kwargs):
        return {}



from .abstract import AbstractModel, XMLElement


class Category(AbstractModel):
    """
    Category model for shop.

    Docs:
    https://yandex.ru/support/partnermarket/elements/categories.html
    """

    __slots__ = ["category_id", "name", "parent_id"]

    def __init__(self, category_id, name, parent_id=None):
        self.category_id = category_id
        self.name = name
        self.parent_id = parent_id

    def create_dict(self, **kwargs) -> dict:
        return dict(id=self.category_id, name=self.name, parent_id=self.parent_id)

    def create_xml(self, **kwargs) -> XMLElement:
        attribs = {"id": self.category_id}

        if self.parent_id:
            attribs["parentId"] = self.parent_id

        el = XMLElement("category", attribs)
        el.text = self.name
        return el

    @staticmethod
    def from_xml(el: XMLElement) -> "Category":
        return Category(category_id=el.attrib["id"], name=el.text, parent_id=el.attrib.get("parentId"))



from django.db import transaction
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.parsers import MultiPartParser

from zs_utils.permissions import StaffPermission
from zs_utils.support import constants, filters, models, serializers, services
from zs_utils.views import CustomModelViewSet


__all__ = [
    "BaseSupportTicketView",
    "BaseSupportTicketMessageView",
    "BaseSupportTicketMessageFileView",
]


class BaseSupportTicketView(CustomModelViewSet):
    """
    View для создания и просмотра тикетов (SupportTicket)
    """
    serializer_class = serializers.CommonSupportTicketSerializer
    filterset_class = filters.SupportTicketFilter
    not_allowed_actions = [
        "destroy",
        "update",
        "partial_update",
    ]

    @classmethod
    def get_support_ticket_service(cls):
        return services.CommonSupportTicketService

    @action(detail=True, methods=["POST"])
    def close(self, request, *args, **kwargs):
        """
        Закрытие тикета
        """
        self.get_support_ticket_service().close_ticket(user=self.get_user(), ticket=self.get_object())
        return self.build_response()

    def create(self, request, *args, **kwargs):
        data = self.get_validated_data(serializer_class=serializers.CommonCreateSupportTicketSerializer)
        message = data.pop("message") if "message" in data else None
        user = self.get_user()
        with transaction.atomic():
            instance = self.get_support_ticket_service().create_ticket(user=user, **data)
            if message:
                self.get_support_ticket_service().create_ticket_message(
                    ticket=instance, sender=user, is_system=False, **message
                )
        return self.build_response(data=self.get_serializer_class()(instance).data, status_code=status.HTTP_201_CREATED)

    @action(detail=True, methods=["POST"], permission_classes=[StaffPermission])
    def take_to_work(self, request, *args, **kwargs):
        """
        Взятие тикета в работу (для staff пользователей)
        """
        self.get_support_ticket_service().take_ticket(manager=self.get_user(), ticket=self.get_object())
        return self.build_response()

    @action(detail=True, methods=["POST"])
    def reopen(self, request, *args, **kwargs):
        """
        Открыть закрытый тикет
        """
        self.get_support_ticket_service().reopen_ticket(user=self.get_user(), ticket=self.get_object())
        return self.build_response()

    @action(detail=True, methods=["POST"])
    def set_viewed(self, request, *args, **kwargs):
        self.get_support_ticket_service().set_ticket_viewed(user=self.get_user(), ticket=self.get_object())
        return self.build_response()

    @action(detail=False, methods=["GET"])
    def get_metadata(self, request, *args, **kwargs):
        """
        Получение метаданных тикета (возможные статусы и тип вопроса)
        """
        data = {
            "status": constants.SUPPORT_TICKET_STATUSES,
            "question_type": constants.SUPPORT_TICKET_QUESTION_TYPES,
        }
        return self.build_response(data=data)


class BaseSupportTicketMessageView(CustomModelViewSet):
    """
    View для создания/удаления/обновления/просмотра сообщений тикета (SupportTicketMessage)
    """
    not_allowed_actions = [
        "update",
        "partial_update",
        "destroy",
    ]

    @classmethod
    def get_support_ticket_service(cls):
        return services.CommonSupportTicketService

    def get_queryset_filter_kwargs(self) -> dict:
        return {"ticket": self.kwargs["ticket_id"]}

    def create(self, request, *args, **kwargs):
        """
        Подстановка тикета SupportTicket.id и пользователя (отправителя) при создании нового сообщения
        """
        data = self.get_validated_data(serializer_class=serializers.CommonCreateSupportTicketMessageSerializer())
        ticket = models.SupportTicket.objects.get(id=kwargs["ticket_id"])
        sender = self.get_user()

        instance = self.get_support_ticket_service().create_ticket_message(
            ticket=ticket, sender=sender, is_system=False, **data,
        )
        return self.build_response(data=self.serializer_class(instance).data, status_code=status.HTTP_201_CREATED)


class BaseSupportTicketMessageFileView(CustomModelViewSet):
    """
    View для создания/удаления/обновления/просмотра файлов сообщений (SupportTicketMessageFile)
    """
    parser_classes = (MultiPartParser,)
    not_allowed_actions = [
        "update",
        "partial_update",
        "destroy",
    ]



from django.utils import timezone


class env_type:
    def __init__(self, config_id, web_endpoint, api_endpoint):
        self.config_id = config_id
        self.web_endpoint = web_endpoint
        self.api_endpoint = api_endpoint


class environment:
    PRODUCTION = env_type(
        "api.ebay.com",
        "https://auth.ebay.com/oauth2/authorize",
        "https://api.ebay.com/identity/v1/oauth2/token",
    )
    SANDBOX = env_type(
        "api.sandbox.ebay.com",
        "https://auth.sandbox.ebay.com/oauth2/authorize",
        "https://api.sandbox.ebay.com/identity/v1/oauth2/token",
    )


class oAuth_token:
    def __init__(
        self,
        error=None,
        access_token: str = None,
        refresh_token: str = None,
        expires_in: int = None,
        refresh_token_expires_in: int = None,
        token_type: str = None,
        **kwargs,
    ):
        """
        access_token_expiry: datetime in UTC
        refresh_token_expiry: datetime in UTC
        """

        self.error = error
        self.access_token = access_token
        self.refresh_token = refresh_token
        self.token_type = token_type

        now = timezone.now()

        if expires_in:
            self.access_token_expiry = now + timezone.timedelta(seconds=expires_in)
        else:
            self.access_token_expiry = None

        if refresh_token_expires_in:
            self.refresh_token_expiry = now + timezone.timedelta(seconds=refresh_token_expires_in)
        else:
            self.refresh_token_expiry = None



from zs_utils.api.shipstation.base_api import ShipstationAPI


class ShipmentShipstationAPIMixin:
    resource_method = "shipments"


class GetShipstationShipmentsListAPI(ShipstationAPI):
    http_method = "GET"
    resource_method = "shipments"


class CreateShipstationLabelAPI(ShipstationAPI):
    """
    Docs: https://www.shipstation.com/docs/api/shipments/create-label/
    """

    http_method = "POST"
    resource_method = "shipments/createlabel"
    allowed_params = [
        "carrierCode",
        "serviceCode",
        "packageCode",
        "confirmation",
        "shipDate",
        "weight",
        "dimensions",
        "shipFrom",
        "shipTo",
        "insuranceOptions",
        "internationalOptions",
        "advancedOptions",
        "testLabel",
    ]


class VoidShipstationLabelAPI(ShipstationAPI):
    http_method = "POST"
    resource_method = "shipments/voidlabel"
    allowed_params = ["shipmentId"]
